;*********************************************************************
;                       Mp3 PuBliC
;                      MP3 HD Player
;
;                                                                
; Ver 1.0                              Coded by Ben Stragnell    /1999 
; Ver 1.1					   Modified by Michael Weston/2001  
;*********************************************************************

		
;***************************************************************************
;*
;* called by timer interrupt every 1024 cycles (around every 200us)
;* We decode SIRCS-protocol signals by measuring the length of LOW pulses
;* observed on A5.
;*
;*     600us = 0                  <300us = glitch - ignore ( <2 ticks)
;*    1200us = 1             300us-900us = 0               (2-4 ticks)
;*    2400us = header       900us-1800us = 1               (5-9 ticks)
;*                               >1800us = header          ( >9 ticks)
;*
;* This code MUST start at 0x1000 !!!!!!
;***************************************************************************/
			org 0x1000
IntHandler			
			swapf		STATUS,0	; get status, don't break flags!
			clrf		STATUS		; reset status, select bank 0
			movwf		STATUS_BKP	; store it
			bsf		LCD_STATUS,LCD_INT	; signal interrupt

			; has last signal been processed?
			btfsc           CUR_STATUS,CS_IRDATAREADY
			goto		ir_done

			; test A5, is it low?
			btfss		PORTA,IR_INPUT
			goto            ir_low

			; time pulse
			movf		IR_LOTIME,0	; get tick count
			addlw           d'254'		; subtract 2
			btfss		STATUS,C	; C -> +ve -> w>=2
			goto		ir_hidone	; if so, discard

			; ok... we have either a bit of data, or a header
			bsf		STATUS,C	; assume '1'
			rrf		IR_DATA_HI,1	; shift data right
			rrf		IR_DATA_LO,1

			; assume data - test for a 0 bit
			movf		IR_LOTIME,0	; get tick count
			addlw		d'251'
			btfss		STATUS,C	; C -> +ve -> w>=5
			bcf		IR_DATA_HI,7    ; if so, clear incoming

			; check for header pulse
			incf            IR_BITCOUNT,1	; get bit count+1
			movf		IR_LOTIME,0	; get tick count
			addlw		d'246'
			btfsc		STATUS,C	; C -> +ve -> w>=10
			clrf		IR_BITCOUNT

			; if bitcount>128, keep at 128
			movf		IR_BITCOUNT,0
			btfsc		IR_BITCOUNT,7
			decf		IR_BITCOUNT,1

			; have we received 15 bits?
			movf		IR_BITCOUNT,0
			xorlw		d'15'
			btfss		STATUS,Z
			goto		ir_hidone

			; yes we have! update IR received value!
			; now we only signal that IR data is received
			; when we get the same signal twice in a row
			bsf		IR_BITCOUNT,7	; prevent until got header

			bcf		STATUS,C
			rrf		IR_DATA_HI,0
			xorwf		IR_RECV_HI,0
			btfss		STATUS,Z
			goto		ir_not_same
			bcf		STATUS,C
			rrf		IR_DATA_LO,0
			xorwf		IR_RECV_LO,0
			btfss		STATUS,Z
			goto		ir_not_same

			bsf		CUR_STATUS,CS_IRDATAREADY

			; restart
ir_restart		clrf		IR_DATA_LO
			clrf		IR_DATA_HI

			; we're done
ir_hidone		clrf		IR_LOTIME
ir_done			goto		ir_complete

ir_low			; increment pulse width
			incf		IR_LOTIME,1
			goto		ir_complete

ir_not_same		; store new value
			bcf		STATUS,C
			rrf		IR_DATA_HI,0
			movwf		IR_RECV_HI
			rrf		IR_DATA_LO,0
			movwf		IR_RECV_LO
			goto		ir_restart


			; consider 20Hz counter
ir_complete		decf		CTR_20HZ,1
			btfss		STATUS,Z
			goto		no_20hz

			; signal timer flag
			bsf		LCD_STATUS,LCD_20HZ_INT

			btfss		CUR_STATUS,CS_STREAMOK
			goto		no_20hz

			; increment time counter
			bsf		STATUS,RP0
			incf		TIME_19TH,1
			movlw		-d'19'
			addwf		TIME_19TH,0
			btfss		STATUS,C
			goto            inc_done
			clrf		TIME_19TH
			incf		TIME_SEC,1
			bsf		LCD_STATUS,LCD_TIMEUPDATE
			movlw		-d'60'
			addwf		TIME_SEC,0
			btfss		STATUS,C
			goto		inc_done
			clrf		TIME_SEC
			incf		TIME_MIN,1
inc_done		bcf		STATUS,RP0

			; restore W,STATUS and return
no_20hz			swapf		STATUS_BKP,0	; get status bkp
			movwf		STATUS		; restore it
			swapf		W_BKP,1		; swap nibbles of W (preserve flags!)
			swapf		W_BKP,0		; unswap nibbles, get W back!
			bcf		INTCON,2	; reset flag
			retfie				; return

;***************************************************************************/
;*
;* ModePaintBottom
;*
;***************************************************************************
;*
;* paint the bottom row of the LCD display according to mode
;*
;***************************************************************************

ModePaintBottom         ; set IO mode
			call		SetIOSRAMWrite
			movlw		SRAM_ADDR_LCD>>8
			movwf		PORTB

			; display track/time?
			btfss		CUR_STATUS,CS_STREAMOK
			goto            mpb_normal
			movf            BUTTON_TIMEOUT,0
			btfsc		STATUS,Z
			goto            ShowTime

mpb_normal		movf		CUR_MODE,0
			addwf		PCL,1
			goto		mpb_volume
			goto		ShowTime
			goto		mpb_trk
			goto		mpb_cd
			goto		mpb_treble
			goto		mpb_bass

;***************************************************************************
;*
;* some handy tables
;*
;***************************************************************************

DecGetLo                addwf		PCL,1
			retlw		d'10'
			retlw		d'100'

ErrorMsgTable		addwf		PCL,1
			dt		"rrE metsyseliF"
SeekMsgTable		addwf		PCL,1
			dt		"DDH cilbuP 3PM"

ModeMsgTable		addwf		PCL,1
			dt		"Vol Pos Trk Cd Trb Bas ",0
;					 0123456789012345678901

StringOffsetTable	addwf		PCL,1
			dt		0,0,0,0,3,7
CursorPosTable		addwf		PCL,1
			dt              0,4,8,d'12',d'12',d'12'
FlagTable		addwf		PCL,1
			dt              1,1,1,1,3,2



;***************************************************************************
;*
;* PCChat
;*
;***************************************************************************
;*
;* PC slave loop... keep receiving and executing commands from the PC.
;*
;***************************************************************************/

PCChat			call		SetIOLatchReadWrite	; latch mode
			call		PCSwap
			movf		PC_RECV_0,0		; get cmd
			addwf		PCL,1			; jump
			goto		cmd_nop
			goto		cmd_writeram
			goto		cmd_readram
			goto		cmd_writeide
			goto		cmd_readide
			goto		cmd_writesector
			goto		cmd_readsector
			goto		cmd_startstream
			goto		cmd_readir
			goto		cmd_idle
			goto		cmd_intcon
			goto		cmd_writesectorepp

;***************************************************************************
;*
;* ButtonHandler
;*
;***************************************************************************
;*
;* If PROC_BUTTON!=0, a button (or IR signal) has been pressed
;* (or has auto-repeated)... process it
;*
;***************************************************************************/

ButtonHandler           ; do we have a button to process?
			movf		PROC_BUTTON,0
			btfsc		STATUS,Z
			goto		ButtonHandlerReturn

			; reset button timeout
			movlw		d'95'			; roughly 5 seconds
			movwf		BUTTON_TIMEOUT

			; store status
			movf		CUR_STATUS,0		; backup status
			movwf		OLD_STATUS

			; too high?
			movf		PROC_BUTTON,0
			addlw		-(VB_MODE+1)
			btfsc		STATUS,C
			goto		bh_done

			; play/stop button?
			movf		PROC_BUTTON,0
			xorlw           VB_PLAYSTOP
			btfsc		STATUS,Z
			goto            bh_playstop

			; mode button?
			movf		PROC_BUTTON,0
			xorlw           VB_MODE
			btfsc		STATUS,Z
			goto            bh_mode

			; test for VB_PLUS or VB_MINUS
			movf		PROC_BUTTON,0
			andlw           b'11111110'
			xorlw		VB_PLUS
			btfss		STATUS,Z
			goto            bh_not_plusminus

			; we have VB_PLUS or VB_MINUS... convert to current
			; mode...
			bcf		STATUS,C
		        rlf		CUR_MODE,0
			addwf           PROC_BUTTON,0
			addlw		-d'12'
			movwf		PROC_BUTTON

			; must be a mode-related button
			; if we're not in the correct mode, set mode
bh_not_plusminus	bcf		STATUS,C
			rrf             PROC_BUTTON,0
			addlw		-d'1'
			movwf		TMP0
			xorwf		CUR_MODE,0
			btfss		STATUS,Z
			goto		bh_modechange

			; ok... process the plus/minus functions here...
			movf		CUR_MODE,0
			addwf		PCL,1
			goto		bh_volchange
			goto		bh_poschange
			goto		bh_trkchange
			goto		bh_cdchange
			goto		bh_trbchange
			goto		bh_baschange

			; increment current mode
bh_mode			incf		CUR_MODE,1
			movlw		d'6'
			xorwf           CUR_MODE,0
			btfsc		STATUS,Z
			clrf		CUR_MODE
			goto		bh_didmodechange

bh_modechange		; set mode
			movf		TMP0,0
			movwf		CUR_MODE
bh_didmodechange	call		ResetScroll
			call		ModePaint
			goto		bh_done

bh_playstop		; are we currently playing?
			btfsc		CUR_STATUS,CS_STREAMOK
			goto            bh_stop

			; start playing
			call		TrackGetStartCluster
			call		TrackGetHeader
			call		TrackSetPos
			call		TrackPrefetch
			call		TrackPlay
			goto		bh_done

bh_stop                 ; stop playing
			call		TrackStop
			goto		bh_done

			; clear button and return
bh_done			clrf		PROC_BUTTON
			goto		ButtonHandlerReturn

bh_trkchange            ; fetch track count from CD header
			call		ResetScroll
			call		TrackStop		; stop playing

			call		GetTrackCount
			movwf		TMP0
			btfsc		PROC_BUTTON,0
			goto            bh_trkdown

			incf		CUR_TRACK,0
			xorwf		TMP0,1
			btfsc		STATUS,Z
			clrw
			movwf           CUR_TRACK
			goto		bh_trkcontinue

bh_trkdown              decf            CUR_TRACK,1
			decf		TMP0,0
			btfsc		CUR_TRACK,7
			movwf		CUR_TRACK

bh_trkcontinue          call		TrackGetStartCluster
			call		TrackGetHeader
			clrf		CUR_INDEX
			call		TrackSetPos
			call		TrackPrefetch
			btfsc		OLD_STATUS,CS_STREAMOK
			call		TrackPlay
			goto		bh_didmodechange

			; CD change up/down
bh_cdchange		call		ResetScroll
			call		TrackStop		; stop playing
			call		GetCdCount
			btfsc		PROC_BUTTON,0
			goto            bh_cddown

			; increment CD count
			incf            CUR_CDLO,1
			btfsc		STATUS,Z
			incf		CUR_CDHI,1

			; did we hit the end?
			movf		CUR_CDHI,0
			xorwf           TMP1,0
			btfss		STATUS,Z
			goto		bh_cdcontinue
			movf		CUR_CDLO,0
			xorwf		TMP0,0
			btfss		STATUS,Z
			goto		bh_cdcontinue
bh_cdoverflow		clrf		CUR_CDHI
			clrf		CUR_CDLO
			goto		bh_cdcontinue

bh_cddown		; decrement CD count
			movlw		d'1'
			subwf		CUR_CDLO,1
			btfss		STATUS,C
			decf		CUR_CDHI,1
			btfss		CUR_CDHI,7
			goto		bh_cdcontinue
			movf		TMP0,0
			movwf		CUR_CDLO
			movf		TMP1,0
			movwf		CUR_CDHI
			goto		bh_cddown

bh_cdcontinue		call		CdGetHeader
			clrf		CUR_TRACK
			clrf		OLD_STATUS
			goto		bh_trkcontinue


bh_poschange		btfsc		PROC_BUTTON,0
			goto		bh_posdown

			incf            CUR_INDEX,1
			call		TrackSetPos
			btfsc           NEXT_CLUSTER_HI,7
			decf		CUR_INDEX,1
			goto		bh_poscontinue

bh_posdown		movf		CUR_INDEX,1
			btfss		STATUS,Z
			decf		CUR_INDEX,1

bh_poscontinue		call		TrackSetPos
			call		TrackPrefetch
			btfsc           OLD_STATUS,CS_STREAMOK
			call		TrackPlay
			goto		bh_done

bh_trbchange		btfsc		LCD_STATUS,LCD_I2CBUSY
			goto		bh_done
			movlw           CUR_TREBLE
			movwf		FSR
			call		AdjustParm
			call		I2CTreble
			goto		bh_didmodechange

bh_baschange		btfsc		LCD_STATUS,LCD_I2CBUSY
			goto		bh_done
			movlw           CUR_BASS
			movwf		FSR
			call		AdjustParm
			call		I2CBass
			goto		bh_didmodechange

bh_volchange		btfsc		LCD_STATUS,LCD_I2CBUSY
			goto		bh_done
			movlw           CUR_VOLUME
			movwf		FSR
			call		AdjustParm
			call		I2CVolume
			goto		bh_didmodechange

;***************************************************************************
;*
;*
;*
;***************************************************************************
;*
;*
;*
;***************************************************************************/

ResetScroll		movlw		-d'5'
			movwf		SCROLL_OFFSET
			bcf             LCD_STATUS,LCD_MSG_SOZ
			return

;***************************************************************************
;*
;* Start
;*
;***************************************************************************
;*
;* Initialise IDE/MCU
;*
;***************************************************************************/

Start_HD
			bsf		STATUS,RP0
			movlw		b'00110111'	; A5=input (MAS), A4=input(IR), A3=enable
			movwf		TRISA		; A2=buttons A1,A0=i2c
			movlw		b'10000000'	; B7 input, B0-6 outputs
			movwf		TRISB
			movlw		b'11111111'	; C and D all inputs
			movwf		TRISC		; C0-C7 inputs (by default)
			movwf		TRISD		; D0-D7 inputs (by default)
			bcf		STATUS,RP0

			; clear memory
			movlw		h'7f'
			movwf		FSR
clrlp			clrf		INDF
			bsf		FSR,7
			clrf		INDF
			bcf		FSR,7
			decf		FSR,1
			movlw		h'1f'
			xorwf           FSR,0
			btfss		STATUS,Z
			goto		clrlp

			; init scroll timer
			movlw		d'1'
			movwf		SCROLL_TIMER

			bsf		SHADOW_C,5 		; backlight

			; enable timer ints every 1024 ticks
			bsf		STATUS,RP0
			movlw		b'00000001'	; pullups on!
			movwf		OPTION_REG
			bcf		STATUS,RP0
			movlw		b'10100000'
			movwf		INTCON

			; init IDE, read filesystem parameters
			call		FilesystemInit
			movlw		d'25'
			movwf		TMP2				; Keep message on LCD for about a second
See_Message
			call		BigDelay
			decfsz	TMP2,1
			goto		See_Message

			; initialise volume and treble settings
			bsf		STATUS,RP0
			movlw		DEFAULT_VOLUME
			movwf		CUR_VOLUME
			movlw		DEFAULT_TREBLE
			movwf		CUR_TREBLE
			movlw		DEFAULT_BASS
			movwf		CUR_BASS
			bcf		STATUS,RP0
			call		I2CVolume
			call		I2CFlush
			call		I2CTreble
			call		I2CFlush
			call		I2CBass
			call		I2CFlush

			; read CUR_CD sector
			call		CdGetHeader
			clrf		CUR_INDEX
			clrf		CUR_TRACK
			call		TrackGetStartCluster

			; fetch track info
			call		TrackGetHeader

			; init screen!
			movlw		MODE_CD
			movwf		CUR_MODE
			call		ResetScroll
			call		ModePaint

			; streaming loop
main_stream_loop	btfsc		CUR_STATUS,CS_TUNEDONE
			goto		TrackFinished

			call		IdeProcessBuffers
			call		Idle
			goto		ButtonHandler
ButtonHandlerReturn

			; msg scroll required?
			btfsc		LCD_STATUS,LCD_SCROLL
			call		ScrollMessage

			; time update required
			btfsc		LCD_STATUS,LCD_TIMEUPDATE
			call		UpdateTime

			; see if I2C needs processing
			call            I2CProcess
			goto		main_stream_loop

;***************************************************************************
;*
;* TimerProcess
;*
;***************************************************************************
;*
;* 20Hz user-interface/keypress processing function... called from Idle
;*
;***************************************************************************/

TimerProcess		; reset flag
			bcf		LCD_STATUS,LCD_20HZ_INT

			; dec timeout
			movf		BUTTON_TIMEOUT,0
			btfss		STATUS,Z
			decf		BUTTON_TIMEOUT,1

			; consider scrolling message every <n> ticks?
			decfsz		SCROLL_TIMER,1
			goto            no_scroll
			movlw		d'6'
			movwf		SCROLL_TIMER
			bsf		LCD_STATUS,LCD_SCROLL
no_scroll

			; do we have any actual physical buttons pressed?
			bsf		STATUS,RP0		; make B inputs
			movlw		b'11111111'
			movwf		TRISB
			bcf		TRISA,BUTTON_CTRL	; =output
			bcf		STATUS,RP0
			bcf		PORTA,BUTTON_CTRL	; button line goes low
			nop					; settle?
			movf		PORTB,0			; read port B again
			xorlw		b'11111111'		; xor!
			movwf		TMP0
			bsf		STATUS,RP0
			bsf		TRISA,BUTTON_CTRL	; =input
			movlw		b'10000000'		; restore outputs
			movwf		TRISB
			bcf		STATUS,RP0
			clrw
			btfsc		TMP0,0
			movlw		VB_PLAYSTOP
			btfsc		TMP0,1
			movlw		VB_MODE
			btfsc		TMP0,2
			movlw		VB_MINUS
			btfsc		TMP0,3
			movlw		VB_PLUS
			iorlw		h'00'
			btfss		STATUS,Z
			goto            got_actual_button

			; got any IR codes?
			btfss		CUR_STATUS,CS_IRDATAREADY
			goto		button_continue
			bcf		CUR_STATUS,CS_IRDATAREADY	; clear flag

			; translate IR code using RAM table (from HD)
			call		SetIOSRAMRead
			movlw           (SRAM_ADDR_IRTABLE>>8)
			movwf		PORTB
			clrf		PORTC
xlate_loop              bsf		PORTA,ENABLE		; read hi byte
			movf		PORTD,0
			bcf		PORTA,ENABLE
			btfsc		STATUS,Z
			goto            button_continue		; 0 = out of codes
			xorwf           IR_RECV_HI,0		; compare
			btfss		STATUS,Z		; match?
			goto		xlate_fail
			incf		PORTC,1			; yes.. move on
			bsf		PORTA,ENABLE		; read lo byte
			movf		PORTD,0
			bcf		PORTA,ENABLE
			xorwf		IR_RECV_LO,0		; compare
			btfss		STATUS,Z		; match?
			goto		xlate_fail
			incf		PORTC,1			; yes.. move on
			bsf		PORTA,ENABLE		; read button code
			movf		PORTD,0
			bcf		PORTA,ENABLE
			movwf		TMP0			; store... done
			movlw		IR_HOLD_TIME		; hold time
			movwf		IR_RELEASE
			goto		got_button		; process button
xlate_fail              movf		PORTC,0			; get lo addr
			andlw		b'11111100'		; mask off low
			addlw		d'4'			; move on
			movwf		PORTC
			btfss		STATUS,Z		; keep going?
			goto		xlate_loop
			clrf		TMP0			; no match

			; process current button status
button_continue		movf		IR_RELEASE,0		; check release counter
			btfss		STATUS,Z		; if zero...
			goto		keep_button
			clrf		CUR_BUTTON 		; reset button id...
			clrf		BUTTON_TIME		;  ...and time counter
			goto		timer_done		; we're done

			; we have a valid button held down
keep_button             decf		IR_RELEASE,1		; dec release counter
			incf            BUTTON_TIME,1		; inc time ctr

			; check for auto-repeat processing
			movlw		d'1'
			xorwf		BUTTON_TIME,0	; ==1?
			btfsc		STATUS,Z
			goto		proc_button
			movlw		d'20'		; ~1 sec hold before repeat
			subwf		BUTTON_TIME,0	; <20?
			btfss		STATUS,C
			goto		timer_done
			bcf		BUTTON_TIME,6
			bsf		BUTTON_TIME,5	; keep between 32 and 64!
			btfsc		BUTTON_TIME,0	; & 1?
			goto		proc_button
			goto		timer_done

			; we need to process a button press
proc_button             movf		CUR_BUTTON,0
			movwf           PROC_BUTTON

			; timer processing complete
timer_done		return


			; compare against previous button
got_actual_button       movwf		TMP0			; store button code
			movlw		d'2'			; set default release time
			movwf		IR_RELEASE		; for physical button

got_button		movf		TMP0,0			; get new button
			btfsc		STATUS,Z		; valid?
			clrf		IR_RELEASE		; if not, cancel!
			xorwf		CUR_BUTTON,0
			btfsc		STATUS,Z		; same as current button?
			goto		same_button
			clrf		BUTTON_TIME		; if not, reset time counter
			movf		TMP0,0			; update current button
			movwf		CUR_BUTTON
same_button             goto		button_continue

;***************************************************************************
;*
;* LCDProcess
;*
;***************************************************************************
;*
;* If we can, copy a byte from the "screen-buffer" to the LCD display
;*
;***************************************************************************/

LCDProcess		; check LCD busy flag
			bcf		LCD_STATUS,LCD_INT
			call		SetIOLCDRead
			bsf		PORTA,ENABLE
			nop
			btfsc		PORTC,7
			goto		lcd_done
			bcf		PORTA,ENABLE

			; have we hit the end of the row?
			btfsc		LCD_OFFSET,5	; LCD_OFFSET>=32?
			goto		last_char

			; blinking cursor?
			btfsc		SCROLL_TIMER,2
			goto		no_cursor

			; may have to put a cursor up there
			movf            LCD_OFFSET,0
			xorwf           LCD_CURSOR,0
			btfss           STATUS,Z
			goto		no_cursor
			clrf		TMP0
			comf		TMP0,1
			incf		LCD_OFFSET,1	; advance ptr
			goto		lcd_gotchar

no_cursor		; busy flag is clear... read byte from RAM
			call		SetIOSRAMRead
			movf		LCD_OFFSET,0
			movwf		PORTC
			movlw		(SRAM_ADDR_LCD>>8)	; 0800=2048 (screen buffer offset)
			movwf		PORTB
			bsf		PORTA,ENABLE	; read byte
			movf		PORTD,0
			bcf		PORTA,ENABLE
			movwf           TMP0		; store byte
			incf		LCD_OFFSET,1	; advance ptr

			; write it to LCD
lcd_gotchar		call		SetIOLCDWrite
			bsf		PORTB,0		; select LCD data
lcd_cont		movf		TMP0,0  	; retrieve char
			movwf		PORTC
			bsf		PORTA,ENABLE
			nop
			goto		lcd_done

last_char		; need to switch rows
			incf            LCD_ROW_CTR,1

			movlw		h'40'		; toggle row
			xorwf		LCD_OFFSET,1
			movlw		h'40'
			andwf		LCD_OFFSET,1
			movf		LCD_OFFSET,0
			iorlw		h'80'		; get command
			movwf		TMP0
			call		SetIOLCDWrite	; select LCD inst-write
			goto		lcd_cont

lcd_done                bcf		PORTA,ENABLE

			return


;***************************************************************************
;*
;* Idle
;*
;***************************************************************************
;*
;* Idle is called frequently, and checks to see if data needs to be spooled
;* out to the MPEG decoder chip.
;* It also checks the LCD status
;*
;***************************************************************************/

Idle            	; copy screen-buffer byte to LCD
			btfsc		LCD_STATUS,LCD_INT
			call		LCDProcess

			; consider the 20Hz user-interface interrupt
			btfsc		LCD_STATUS,LCD_20HZ_INT
			call		TimerProcess

			; streaming enabled?
			btfss		CUR_STATUS,CS_STREAMOK
			return

			; main MAS streaming loop
masloop
			; test DEMAND line
			btfss		PORTA,MAS_DEMAND
			return

			; ensure the buffer contains some data
			movf		SRAM_COUNT_LO,0
			iorwf		SRAM_COUNT_HI,0
			btfsc		STATUS,Z
			goto		buffer_empty

			; send 4 bytes to the PC
			movlw		d'4'
			movwf		CTR

stream_lp		call		SetIOSRAMRead
			movf		SRAM_READ_HI,0
			andlw		h'7f'
			iorlw		h'40'
			movwf		PORTB
			movf		SRAM_READ_LO,0
			movwf		PORTC
			bsf		PORTA,ENABLE
			movf		PORTD,0
			bcf		PORTA,ENABLE

			incf		SRAM_READ_LO,1
			btfsc		STATUS,Z
			incf		SRAM_READ_HI,1

			movwf		TMP0			; store byte

			call		SetIOLatchReadWrite
			movlw		b'11100000'
			movwf		PORTC
			bsf		PORTA,ENABLE

			bsf		PORTC,7			; data
			btfss           TMP0,7
			bcf		PORTC,7
			bcf		PORTC,6			; lower clock
			bsf		PORTC,6

			bsf		PORTC,7			; data
			btfss           TMP0,6
			bcf		PORTC,7
			bcf		PORTC,6			; lower clock
			bsf		PORTC,6

			bsf		PORTC,7			; data
			btfss           TMP0,5
			bcf		PORTC,7
			bcf		PORTC,6			; lower clock
			bsf		PORTC,6

			bsf		PORTC,7			; data
			btfss           TMP0,4
			bcf		PORTC,7
			bcf		PORTC,6			; lower clock
			bsf		PORTC,6

			bsf		PORTC,7			; data
			btfss           TMP0,3
			bcf		PORTC,7
			bcf		PORTC,6			; lower clock
			bsf		PORTC,6

			bsf		PORTC,7			; data
			btfss           TMP0,2
			bcf		PORTC,7
			bcf		PORTC,6			; lower clock
			bsf		PORTC,6

			bsf		PORTC,7			; data
			btfss           TMP0,1
			bcf		PORTC,7
			bcf		PORTC,6			; lower clock
			bsf		PORTC,6

			bsf		PORTC,7			; data
			btfss           TMP0,0
			bcf		PORTC,7
			bcf		PORTC,6			; lower clock
			bsf		PORTC,6

			bcf		PORTA,ENABLE

			decfsz		CTR,1
			goto		stream_lp

			; fix counter
			movlw		d'4'
			subwf		SRAM_COUNT_LO,1
			btfss		STATUS,C
			decf		SRAM_COUNT_HI,1

			; keep going as long as we can
			goto		masloop

buffer_empty		; the buffer is out of data - have we reached the
			; end of the song???
			btfss		CUR_STATUS,CS_LASTDATA
			return

			; if so, we're done playing
			bcf		CUR_STATUS,CS_STREAMOK
			bsf		CUR_STATUS,CS_TUNEDONE
			return


;***************************************************************************
;*
;* PCSwap
;*
;***************************************************************************
;*
;* Receive 16-bits from the PC into PC_RECV_0 and PC_RECV_1. Send PC_SEND
;* (8-bits) back to the PC. Uses a simple parallel port based synchronous
;* protocol.
;* assumes we're in PC latch IO mode.
;*
;***************************************************************************/

PCSwap
			; recover previous PORTC value
			movf		SHADOW_C,0
			movwf		PORTC
			bsf		PORTA,ENABLE

			; wait for PC signal
swaploop		btfsc		PORTB,7			; wait for low
			goto		swaploop

			movf		PORTD,0			; read/store data
			movwf		PC_RECV_0

			movf		PC_SEND,0
			andlw		b'00001111'		; echo low 4-bits
			iorlw		b'11100000'
			movwf		PORTC
			bsf		PORTC,4

swaploop2		btfss		PORTB,7			; wait for high
			goto		swaploop2

			movf		PORTD,0			; read/store data
			movwf		PC_RECV_1

			swapf		PC_SEND,0
			andlw		b'00001111'
			iorlw		b'11110000'
			movwf		PORTC
			bcf		PORTC,4
			bcf		PORTA,ENABLE

			movwf		SHADOW_C
			bcf		SHADOW_C,4

			return

;***************************************************************************
;*
;* various slave-mode commands
;*
;***************************************************************************
;*
;*
;*
;***************************************************************************/

			; echo byte back to PC
cmd_nop			movf		PC_RECV_1,0
			movwf		PC_SEND
			goto		PCChat

			; receive address, write to SRAM
cmd_writeram		movf		PC_RECV_1,0		; get data
			movwf		TMP0			; store it
			call		PCSwap			; get addr
			call		SetIOSRAMWrite		; set up for WR
			movf		TMP0,0			; get DATA
			movwf		PORTD			; set it
			movf		PC_RECV_0,0		; get HI addr
			movwf		PORTB			; set it
			movf		PC_RECV_1,0		; set LO addr
			movwf           PORTC			; set it
			bsf		PORTA,ENABLE		; do the write
			bcf		PORTA,ENABLE
			goto		PCChat			; continue

			; receive address, read from SRAM
cmd_readram		call		PCSwap			; get addr
			call		SetIOSRAMRead		; set up for RD
			movf		PC_RECV_0,0		; get HI addr
			movwf		PORTB			; set it
			movf		PC_RECV_1,0		; set LO addr
			movwf           PORTC			; set it
			bsf		PORTA,ENABLE		; do the write
			movf		PORTD,0			; read data
			movwf		PC_SEND			; send next time
			bcf		PORTA,ENABLE
			goto		PCChat			; continue

			; write IDE controller reg
cmd_writeide		movf		PC_RECV_1,0		; get addr
			movwf		PORTB			; store it
			call		PCSwap       		; get data
			call		SetIOIDEWrite		; set up for WR
			movf		PC_RECV_0,0		; get data hi
			movwf		PORTD			; set it
			movf		PC_RECV_1,0		; get data lo
			movwf		PORTC			; set it
			bsf		PORTA,ENABLE		; do the write
			nop					; safety
			bcf		PORTA,ENABLE
			goto		PCChat

			; read IDE controller reg
cmd_readide		movf		PC_RECV_1,0		; get addr
			movwf		PORTB			; store it
			call		SetIOIDERead		; set up for WR
			bsf		PORTA,ENABLE
			nop					; safety
			movf		PORTC,0			; get data lo
			movwf		TMP0
			movf		PORTD,0			; get data hi
			movwf		PC_SEND			; set for send
			bcf		PORTA,ENABLE
			call		SetIOLatchReadWrite	; latch mode
			call		PCSwap			; send hi data
			movf		TMP0,0			; retrieve lo data
			movwf		PC_SEND			; set for send
			goto		PCChat			; continue

			; debug - start streaming
cmd_startstream		call		FilesystemInit		; init FS
			clrf		CUR_CLUSTER_LO
			clrf		CUR_CLUSTER_MID
			clrf		CUR_CLUSTER_HI
			call		TrackGetHeader
			clrf		CUR_INDEX
			call		TrackSetPos
			call		TrackPrefetch
			call		TrackPlay
			goto		main_stream_loop	; start streaming

cmd_readir		; read last IR value
			btfss		CUR_STATUS,CS_IRDATAREADY
			goto		noirdata
			movf		IR_RECV_LO,0
			movwf		PC_SEND
			call		PCSwap
			movf		IR_RECV_HI,0
			movwf		PC_SEND
			call		PCSwap
			clrf		IR_RECV_LO
			clrf		IR_RECV_HI
			bcf		CUR_STATUS,CS_IRDATAREADY
			goto		PCChat
noirdata		clrf		PC_SEND
			call		PCSwap
			call		PCSwap
			goto		PCChat

cmd_idle		call		Idle
			goto		PCChat

cmd_intcon              movf		PC_RECV_1,0
			movwf		INTCON
			goto		PCChat

;***************************************************************************
;*
;* cmd_readsector
;*
;***************************************************************************
;*
;* Transfer 512 bytes from the IDE_DATA register to the PC.
;*
;***************************************************************************/

cmd_readsector
			movlw		d'16'			; 16x32=512
			movwf		TMP0
			movlw		IDE_REG_DATA		; point to DATA
			movwf		PORTB

ReadSectorLoop		movlw		TRANS_BUFFER-1		; init ptr
			movwf           FSR
			movlw		d'16'			; init ctr
			movwf		CTR
			call		SetIOIDERead		; init IO

ir32lp			bsf		PORTA,ENABLE		; enable RD
			incf		FSR,1
			movf		PORTC,0
			movwf		INDF
			incf		FSR,1
			movf		PORTD,0
			movwf		INDF
			bcf		PORTA,ENABLE
			decfsz		CTR,1
			goto		ir32lp

			; now send these bytes to the PC
			call		SetIOLatchReadWrite	; setup IO
			movlw		d'32'			; 32 bytes
			movwf		CTR
			movlw		TRANS_BUFFER		; set ptr
			movwf		FSR

			; restore previous PORTC value
			movf		SHADOW_C,0
			movwf		PORTC

			bsf		PORTA,ENABLE		; enable latch

s32lp			movf		INDF,0			; get data
			andlw		b'00001111'		; mask off bits
			iorlw		b'11100000'		; keep mas bits high
s32waitlo               btfsc		PORTB,7			; wait for LO
			goto		s32waitlo
			movwf		PORTC			; write data
			bsf		PORTC,4
			swapf		INDF,0			; get data
			incf		FSR,1			; inc ptr
			andlw		b'00001111'		; mask off bits
			iorlw		b'11110000'		; keep ack,mas high
s32waithi               btfss		PORTB,7			; wait for HI
			goto		s32waithi
			movwf		PORTC			; write data
			bcf		PORTC,4
			decfsz          CTR,1			; dec ctr
			goto            s32lp			; repeat until done

			bcf		PORTA,ENABLE		; disable latch

			movwf		SHADOW_C		; store last value
			bcf		SHADOW_C,4

			decfsz		TMP0,1			; done all blocks?
			goto		ReadSectorLoop

			goto		PCChat

;***************************************************************************
;*
;* cmd_writesector
;*
;***************************************************************************
;*
;* Transfer 512 bytes from the PC parallel port to the IDE_DATA register
;*
;***************************************************************************/

cmd_writesector
			movlw		d'16'			; 16x32=512
			movwf		TMP0
			movlw		IDE_REG_DATA		; point to DATA
			movwf		PORTB

			; receive 32 bytes from the PC into TRANS_BUFFER
WriteSectorLoop		movlw		TRANS_BUFFER		; set ptr
			movwf		FSR
			call		SetIOLatchReadWrite	; prepare latch
			movlw		d'16'			; set counter
			movwf		CTR
			clrf		PORTC			; clear response

			movf		SHADOW_C,0		; retrieve PORTC value
			movwf		PORTC

			bsf		PORTA,ENABLE		; enable latch

r32lp
r32waitlo               btfsc		PORTB,7			; wait for LO
			goto		r32waitlo
			movf		PORTD,0			; read data
			movwf		INDF			; store it
			incf		FSR,1			; advance ptr
			bsf		PORTC,4			; acknowledge

r32waithi               btfss		PORTB,7			; wait for HI
			goto		r32waithi
			movf		PORTD,0			; read data
			movwf		INDF			; store it
			incf		FSR,1			; advance ptr
			bcf		PORTC,4			; acknowledge

			decfsz		CTR,1			; done yet?
			goto		r32lp			; if not, continue

			bcf		PORTA,ENABLE

			; write 32 bytes from TRANS_BUFFER to the IDE data port
			movlw		TRANS_BUFFER		; set ptr
			movwf		FSR
			call		SetIOIDEWrite		; set IO mode
			movlw		d'16'			; set ctr
			movwf		CTR
iw32lp                  movf		INDF,0			; get LO data
			movwf           PORTC			; send to port
			incf		FSR,1			; advance ptr
			movf		INDF,0			; get HI data
			movwf		PORTD			; set HI data
			bsf		PORTA,ENABLE		; enable WR
			incf		FSR,1			; advance ptr
			bcf		PORTA,ENABLE		; disable WR
			decfsz		CTR,1			; repeat 16 times
			goto		iw32lp

			decfsz		TMP0,1			; repeat 16 blocks
			goto            WriteSectorLoop

			goto		PCChat


;***************************************************************************
;*
;* cmd_writesectorepp
;*
;***************************************************************************
;*
;* Transfer 512 bytes from the PC parallel port to the IDE_DATA register
;* Try using EPP mode!
;*
;***************************************************************************/

cmd_writesectorepp
			; 256 16-bit words
			clrf		CTR

			; init portb, since that isn't going to change
			movlw		IDE_REG_DATA
			movwf		PORTB

			; portc is always output
			bsf		STATUS,RP0
			clrf		TRISC
			clrf		TRISD
			bcf		STATUS,RP0

epp_loop		; select the latch, enable it
			bcf		PORTA,ENABLE
			movlw		DEMUX_LATCH_RDWR
			movwf		PORTE
			bsf		STATUS,RP0
			decf		TRISD,1			; trisd->0xff
			bcf		STATUS,RP0
			bcf		PORTC,4
			bsf		PORTA,ENABLE

			; read first byte
epp_waitlo		btfsc		PORTB,7			; wait for DataStrobe lo
			goto		epp_waitlo
			movf		PORTD,0			; read data
			bsf		PORTC,4			; BUSY hi
			movwf		TMP0			; store it
epp_waithi		btfss		PORTB,7			; wait for DataStrobe hi
			goto		epp_waithi
			bcf		PORTC,4			; BUSY lo

			; read second byte
epp_waitlo2		btfsc		PORTB,7			; wait for DataStrobe lo
			goto		epp_waitlo2
			movf		PORTD,0			; read data
			movwf		PORTD			; store it back in PORTD
			bsf		PORTC,4			; BUSY hi
epp_waithi2		btfss		PORTB,7			; wait for DataStrobe hi
			goto		epp_waithi2
			bcf		PORTC,4			; prepare BUSY lo

			bcf		PORTA,ENABLE		; disable latch

			; switch over to IDE write
			movlw		DEMUX_IDE_WR
			movwf		PORTE
			bsf		STATUS,RP0
			clrf		TRISD
			bcf		STATUS,RP0

			; set data
			movf		TMP0,0
			movwf		PORTC

			; write it
			bsf		PORTA,ENABLE

			; rinse and repeat
			decfsz		CTR,1
			goto		epp_loop

			bcf		PORTA,ENABLE

			; we're done
			goto		PCChat

;***************************************************************************
;*
;* IdeReadSectors
;*
;***************************************************************************
;*
;* Read SEC_CT sectors at IDE_SEC0..3 into SRAM, at SRAM_HI:SRAM_LO
;* Idle is called every "block".
;*
;***************************************************************************/

IdeReadSectors		; wait for IDE not busy
			call		WaitForIDEReady

			; set up IDE_SEC0-3
			call		SetIOIDEWrite
			call		SectorSetup

			; write READ SECTORS command
			movlw		IDE_REG_CMD
			movwf		PORTB
			movlw		h'21'
			movwf		PORTC
			bsf		PORTA,ENABLE
			nop					; safety
			bcf		PORTA,ENABLE

			; we want SEC_CT * 16 blocks
			rlf		SEC_CT,1
			rlf		SEC_CT,1
			rlf		SEC_CT,1
			rlf		SEC_CT,0
			andlw		b'11110000'
			movwf		CUR_BLOCK

			; wait for data ready
			call		WaitForIDEData

			; read CUR_BLOCK blocks of 32-bytes (16-words) each
blocklp			call		Idle

			; every whole sector (16 blocks), check status
			movf		CUR_BLOCK,0
			andlw		b'00001111'
			btfsc		STATUS,Z
			call		WaitForIDEReady

			; read 16 words from IDE into TMP1:TMP0
			movlw		d'16'
			movwf		TMP2
			call		IDETransferWords

			; inc RAM ptr by 32 bytes
			movlw		d'32'
			addwf		SRAM_LO,1
			btfsc		STATUS,C
			incf		SRAM_HI,1

			decfsz		CUR_BLOCK,1
			goto		blocklp

			return


;***************************************************************************
;*
;* IdeGetStatus
;*
;***************************************************************************
;*
;* Read the IDE_STATUS register into W
;* (ssumes we're already in IDE read mode)
;*
;***************************************************************************/

IdeGetStatus		movlw		IDE_REG_STATUS
			movwf		PORTB
			bsf		PORTA,ENABLE
			nop					; safety
			movf		PORTC,0
			bcf		PORTA,ENABLE
			return

;***************************************************************************
;*
;* IDETransferWords
;*
;***************************************************************************
;*
;* Read TMP2 16-bit words into TRANS_BUFFER, then transfer to SRAM_HI:SRAM_LO
;* (assumes 32-byte alignment to avoid wrapping check)
;*
;***************************************************************************/

IDETransferWords
			call		SetIOIDERead
			movlw		IDE_REG_DATA
			movwf		PORTB

			; set buffer and counter
			movlw		TRANS_BUFFER-1
			movwf		FSR
			movf		TMP2,0
			movwf		CTR

idereadlp		bsf		PORTA,ENABLE
			incf		FSR,1
			movf		PORTC,0
			movwf		INDF
			incf		FSR,1
			movf		PORTD,0
			movwf		INDF
			bcf		PORTA,ENABLE
			decfsz		CTR,1
			goto		idereadlp

			; write CTR * 2 bytes
			bcf		STATUS,C
			rlf		TMP2,0
			movwf		CTR

			; set up address
			call		SetIOSRAMWrite
			movf            SRAM_HI,0
			movwf		PORTB
			movf		SRAM_LO,0
			movwf		PORTC

			movlw		TRANS_BUFFER
			movwf		FSR

sramwritelp		movf		INDF,0
			movwf		PORTD
			bsf		PORTA,ENABLE
			bcf		PORTA,ENABLE
			incf		FSR,1
			incf		PORTC,1
			decfsz		CTR,1
			goto		sramwritelp

			return

;***************************************************************************
;*
;* WaitForIDEReady
;*
;***************************************************************************
;*
;* Check the status register until !BSY and DRDY. Calls Idle while waiting.
;*
;***************************************************************************/

WaitForIDEReady		call		Idle
			call		SetIOIDERead
			call		IdeGetStatus
			movwf		TMP0
			btfsc           TMP0,7
			goto		WaitForIDEReady
			btfss		TMP0,6
			goto		WaitForIDEReady
			return

;***************************************************************************
;*
;* WaitForIDEData
;*
;***************************************************************************
;*
;* Wait for the IDE drive to return with some data. Calls Idle while waiting.
;*
;***************************************************************************/

WaitForIDEData		call		Idle
			call		SetIOIDERead
			call		IdeGetStatus
			movwf		TMP0
			andlw		b'11001000'
			xorlw		b'01001000'
			btfss		STATUS,Z
			goto		WaitForIDEData
			return

;***************************************************************************
;*
;* SectorSetup
;*
;***************************************************************************
;*
;* Write IDE_SEC0..3 to the appropriate IDE sector registers
;*
;***************************************************************************/

SectorSetup             clrf		PORTD
			movlw		d'3'
			movwf		CTR
			movlw		IDE_REG_SEC0
			movwf		PORTB
			movlw		IDE_SEC0
			movwf		FSR
secregloop              movf		INDF,0
			movwf		PORTC
			bsf		PORTA,ENABLE
			nop					; safety
			bcf		PORTA,ENABLE
			incf		FSR,1
			incf		PORTB,1
			decfsz		CTR,1
			goto		secregloop

			; set dev/LBA
			movf		INDF,0
			andlw		h'0f'
			iorlw		h'e0'
			movwf		PORTC
			bsf		PORTA,ENABLE
			nop					; safety
			bcf		PORTA,ENABLE

			; write sector count
			movlw		IDE_REG_SECTORCOUNT
			movwf		PORTB
			movf		SEC_CT,0
			movwf		PORTC
			bsf		PORTA,ENABLE
			nop					; safety
			bcf		PORTA,ENABLE

			return

;***************************************************************************
;*
;* SetIOLCDWrite/Read
;*
;***************************************************************************
;*
;* Set up ports for reading/writing to the LCD
;*
;***************************************************************************/

SetIOLCDWrite           clrf		PORTB
			clrw
			goto		SetIOLCDCommon
SetIOLCDRead		movlw           d'2'
			movwf		PORTB
			movlw		h'ff'
SetIOLCDCommon		bsf		STATUS,RP0
			movwf           TRISC
			bcf		STATUS,RP0
			movlw		DEMUX_LCD
			movwf		PORTE
			return

;***************************************************************************
;*
;* SetIOIDERead/Write
;*
;***************************************************************************
;*
;* Set PORTA and ports C/D for IDE reading/writing
;*
;***************************************************************************/

SetIOIDERead		movlw		DEMUX_IDE_RD
			movwf		PORTE
			movlw           h'ff'
			goto		SetIOIDECommon
SetIOIDEWrite		movlw		DEMUX_IDE_WR
			movwf		PORTE
			clrw
SetIOIDECommon		bsf             STATUS,RP0
			movwf           TRISC
			movwf		TRISD
			bcf		STATUS,RP0
			return

;***************************************************************************
;*
;* SetIOSRAMRead/Write
;*
;***************************************************************************
;*
;* Set PORTA and ports C/D for SRAM reading/writing
;*
;***************************************************************************/

SetIOSRAMRead		movlw		DEMUX_SRAM_RD
			movwf		PORTE
			movlw           h'ff'
			goto		SetIOSRAMCommon
SetIOSRAMWrite		movlw		DEMUX_SRAM_WR
			movwf		PORTE
			clrw
SetIOSRAMCommon		bsf             STATUS,RP0
			clrf		TRISC
			movwf		TRISD
			bcf		STATUS,RP0
			return

;***************************************************************************
;*
;* SetIOSRAMWrite
;*
;***************************************************************************
;*
;* Set PORTA and ports C/D for reading/writing to the PC latch/buffer
;*
;***************************************************************************/

SetIOLatchReadWrite	movlw		DEMUX_LATCH_RDWR
			movwf		PORTE
			bsf		STATUS,RP0
			clrf		TRISC
			movlw		h'ff'
			movwf		TRISD
			bcf		STATUS,RP0
			return

;***************************************************************************
;*
;* ClusterToSector
;*
;***************************************************************************
;*
;* Convert a cluster address in CUR_CLUSTER to an LBA sector address in
;* IDE_SEC0..3, using the DATA_START values.
;*
;***************************************************************************/

ClusterToSector
			; ((CUR_CLUSTER+DATA_START)<<4)
			; start by getting CUR_CLUSTER+DATA_START
			movf		CUR_CLUSTER_LO,0	; add lo bytes
			addwf		DATA_START_LO,0
			movwf		IDE_SEC0
			movf            DATA_START_MID,0	; add mid bytes
			movwf		IDE_SEC1
			movf		CUR_CLUSTER_MID,0
			btfsc		STATUS,C
			incfsz		CUR_CLUSTER_MID,0
			addwf           IDE_SEC1,1
			movf            DATA_START_HI,0		; add hi bytes
			movwf		IDE_SEC2
			movf		CUR_CLUSTER_HI,0
			btfsc		STATUS,C
			incfsz		CUR_CLUSTER_HI,0
			addwf           IDE_SEC2,1

			; now multiply by 16
			clrf		IDE_SEC3
			movlw		d'4'
			movwf		CTR
shiftlp			rlf		IDE_SEC0,1
			rlf		IDE_SEC1,1
			rlf		IDE_SEC2,1
			rlf		IDE_SEC3,1
			decfsz		CTR,1
			goto		shiftlp
			movlw		h'f0'
			andwf		IDE_SEC0,1

			; phew
			return


;***************************************************************************
;*
;* BigDelay
;*
;***************************************************************************
;*
;* Delay for a while...
;*
;***************************************************************************/

BigDelay		clrf		TMP1

waitlp			clrw
			call		Delay
			decfsz		TMP1,1
			goto		waitlp

			return

;***************************************************************************
;*
;* FilesystemInit
;*
;***************************************************************************
;*
;* Reset the IDE drive, and read the first sector into SRAM at $0000
;* Read the FAT_START and DATA_START values.
;*
;***************************************************************************/

FilesystemInit		; software reset drive
			call		SetIOIDEWrite		; set dev/head
			movlw		IDE_REG_DEVHEAD
			movwf           PORTB
			clrf		PORTD
			movlw		h'e0'
			movwf		PORTC
			bsf		PORTA,ENABLE
			nop
			bcf		PORTA,ENABLE

			;movlw		IDE_REG_CTRL		; raise srst
			;movwf           PORTB
			;movlw		h'06'
			;movwf		PORTC
			;bsf		PORTA,ENABLE
			;nop
			;bcf		PORTA,ENABLE

			;movlw		DEMUX_IDE_WR
			;movwf		PORTE

			;movlw		h'02'			; lower srst
			;movwf		PORTC
			;bsf		PORTA,ENABLE
			;nop
			;bcf		PORTA,ENABLE

			call		BigDelay
			call		BigDelay

			; display startup message
			call		LCDClearAll
			movlw		d'15'
			movwf		CTR
			clrf		PORTC
seek_lp			decf		CTR,0
			call            SeekMsgTable
			call		PrintChar
			decfsz		CTR,1
			goto		seek_lp

			; read the first 2 sectors from the drive
			clrf		IDE_SEC0
			clrf		IDE_SEC1
			clrf		IDE_SEC2
			clrf		IDE_SEC3
			movlw		d'2'
			movwf		SEC_CT
			movlw		(SRAM_ADDR_FSHDR>>8)
			movwf		SRAM_HI
			clrf		SRAM_LO
			call		IdeReadSectors
			call		VerifyFs

			; extract the relevant information
			; (should check for validity!)
			call		SetIOSRAMRead
			movlw		(SRAM_ADDR_FSHDR>>8)
			movwf		PORTB

			movlw		HDROFFSET_FATSTART
			movwf		PORTC
			movlw		FAT_START_0
			movwf		FSR
			call		ReadCluster
			bsf		PORTA,ENABLE		; read 4th byte
			movf		PORTD,0
			bcf		PORTA,ENABLE
			movwf		INDF

			movlw		HDROFFSET_DATASTART
			movwf		PORTC
			movlw		DATA_START_LO
			movwf		FSR
			call		ReadCluster

			; invalidate cached FAT sector
			movlw		h'ff'
			movwf		CUR_FAT_CACHED_LO
			movwf		CUR_FAT_CACHED_HI

			return


;***************************************************************************
;*
;* Delay
;*
;***************************************************************************
;*
;* Generic delay routine - delay for 4+3*W (including call/return)
;*
;***************************************************************************/

Delay			movwf		TMP0
delay_lp		decfsz		TMP0,1
			goto		delay_lp
			return

;***************************************************************************
;*
;* CdGetHeader
;*
;***************************************************************************
;*
;* Compute the address of CUR_CD, and copy the sector into SRAM
;*
;***************************************************************************/

CdGetHeader             ; get cdStart from filesystem header
			call		SetIOSRAMRead
			movlw           (SRAM_ADDR_FSHDR>>8)
			movwf		PORTB
			movlw		HDROFFSET_CDSTART
			movwf		PORTC
			movlw		IDE_SEC0
			movwf		FSR
			call		ReadCluster
			bsf		PORTA,ENABLE		; read 4th byte
			movf		PORTD,0
			bcf		PORTA,ENABLE
			movwf		INDF

			; add CUR_CD to IDE_SEC
			movf		CUR_CDLO,0
			addwf           IDE_SEC0,1
			movf		CUR_CDHI,0
			btfsc		STATUS,C
			incfsz		CUR_CDHI,0
			addwf           IDE_SEC1,1
			bcf		STATUS,Z
			btfsc		STATUS,C
			incf            IDE_SEC2,1
			btfsc		STATUS,Z
			incf		IDE_SEC3,1

			; read the sector
			movlw		(SRAM_ADDR_CDHDR>>8)
			movwf		SRAM_HI
			clrf		SRAM_LO
			movlw		d'1'
			movwf		SEC_CT
			call		IdeReadSectors
			call            VerifyCd

			return

;***************************************************************************
;*
;* TrackGetStartCluster
;*
;***************************************************************************
;*
;* Use CUR_TRACK and the loaded CD to get the track start cluster
;*
;***************************************************************************/

TrackGetStartCluster    ; want to read from SRAM
			call		SetIOSRAMRead

			; set up address
			rlf		CUR_TRACK,0		; ct*2
			addlw           (CDHDROFFSET_TRKCLUSTERS/2)
			movwf		PORTC
			rlf		PORTC,1			; ct*4+112
			rlf		PORTB,0			; hi part
			andlw		b'00000001'
			iorlw           (SRAM_ADDR_CDHDR>>8)
			movwf		PORTB
			movlw		b'11111100'
			andwf		PORTC,1

			; read 3 bytes
			movlw		CUR_CLUSTER_LO
			movwf		FSR
			call		ReadCluster

			return

;***************************************************************************
;*
;* TrackGetHeader
;*
;***************************************************************************
;*
;* Copy 1024 bytes of header information from the cluster at CUR_CLUSTER
;* into SRAM at address 1024.
;*
;***************************************************************************/

TrackGetHeader		call		ClusterToSector		; convert
			movlw		(SRAM_ADDR_TRKHDR>>8)
			movwf		SRAM_HI			; set sram ptr
			clrf		SRAM_LO
			movlw		d'2'			; set sector count
			movwf		SEC_CT
			call		IdeReadSectors		; read 'em
			call            VerifyTrk
			return


;***************************************************************************
;*
;* TrackReadIndex
;*
;***************************************************************************
;*
;* Read the CUR_INDEX and CUR_INDEX+1 index clusters into CUR_CLUSTER
;* and NEXT_CLUSTER
;*
;***************************************************************************/

TrackReadIndex		; want to read cluster no. from SRAM
			call		SetIOSRAMRead

			; get (CUR_INDEX * 5) + 1024 + 128
			movlw		d'1'
			movwf		PORTB		; 256
			movf		CUR_INDEX,0
			addlw		d'32'		; add 32 clocks=128 bytes
			movwf		PORTC		; set lo
			rlf		PORTC,1
			rlf		PORTB,1
			rlf		PORTC,1
			rlf		PORTB,1
			movf		PORTC,0
			andlw		b'11111100'
			addwf           CUR_INDEX,0
			movwf           PORTC
			btfsc		STATUS,C
			incf		PORTB,1

			; read data into CUR_CLUSTER
			movlw		CUR_CLUSTER_LO
			movwf		FSR
			call		ReadCluster

			; read next 3 bytes into TIME vars (only 2 used!)
			movlw		TIME_19TH
			movwf		FSR
			call		ReadCluster

			; dec ptr by 1 byte, wrap on PORTB
			movf		PORTC,0
			btfsc		STATUS,Z
			decf		PORTB,1
			decf		PORTC,1

			; read data into NEXT_CLUSTER
			movlw		NEXT_CLUSTER_LO
			movwf		FSR
			call		ReadCluster

			; extract time values
			bsf		STATUS,RP0
			movf            TIME_SEC,0		; ssmmmmmm
			andlw           b'00111111'
			movwf           TIME_MIN		; store minutes
			rrf		TIME_SEC,1		; ?ssmmmmm
			rrf		TIME_SEC,0		; ??ssmmmm
			andlw		b'00110000'
			movwf		TIME_SEC
			swapf		TIME_19TH,0		; ffffssss
			andlw		b'00001111'
			iorwf           TIME_SEC,1		; 00ssssss
			movlw		b'00001111'
			andwf           TIME_19TH,1		; 0000ffff
			rlf		TIME_19TH,1		; 000ffff0
			bcf		STATUS,RP0

			bsf		LCD_STATUS,LCD_TIMEUPDATE

			return


;***************************************************************************
;*
;* TrackSetPos
;*
;***************************************************************************
;*
;* Set the current track to the Nth index block
;*
;***************************************************************************/

TrackSetPos		; ensure we're not playing
			call		TrackStop

			; reset end-of-data flags
			bcf		CUR_STATUS,CS_TUNEDONE
			bcf		CUR_STATUS,CS_LASTDATA

			; read the index clusters into CUR_CLUSTER and
			; NEXT_CLUSTER
			call		TrackReadIndex

			; is this the first?
			bcf		CUR_STATUS,CS_SKIPHEADER
			movf		CUR_INDEX,0
			btfsc		STATUS,Z
			bsf		CUR_STATUS,CS_SKIPHEADER

			; done
			return


;***************************************************************************
;*
;* TrackStop
;*
;***************************************************************************
;*
;* Stop playing
;*
;***************************************************************************/

TrackStop		; reset flags, disable streaming
			bcf		CUR_STATUS,CS_STREAMOK
			return

;***************************************************************************
;*
;* TrackPlay
;*
;***************************************************************************
;*
;* Start/resume playing
;*
;***************************************************************************/

TrackPlay		bsf		CUR_STATUS,CS_STREAMOK
			return


;***************************************************************************
;*
;* TrackPrefetch
;*
;***************************************************************************
;*
;* Start playing the current track, at the CUR_CLUSTER pos...
;* If CS_SKIPHEADER is set, skip the 1024-byte header
;*
;***************************************************************************/

TrackPrefetch           ; clear some flags
			bcf		CUR_STATUS,CS_TUNEDONE
			bcf		CUR_STATUS,CS_LASTDATA

			; reset SRAM write pointers and byte counters
			clrf		SRAM_WRITE_LO
			clrf		SRAM_WRITE_HI
			clrf		SRAM_COUNT_LO
			clrf		SRAM_COUNT_HI

			; fetch the first 8K
			call		IdeProcessBuffers

			; should we skip the first 1024 bytes?
			; HACK!!! YES - ALWAYS! ENSURE THAT THE SRAM'S AS
			; FULL AS POSSIBLE!!!
;			btfss		CUR_STATUS,CS_SKIPHEADER
;			goto		no_skip

			; if so, adjust ptrs
			movlw		h'20'-h'04'
			movwf		SRAM_COUNT_HI
			clrf		SRAM_READ_LO
			movlw           h'04'
			movwf		SRAM_READ_HI

			; reset flag
			bcf		CUR_STATUS,CS_SKIPHEADER

			; read the next 8K (assuming there is some!)
no_skip			btfss		CUR_STATUS,CS_LASTDATA
			call		IdeProcessBuffers

			; we're done
			return



;***************************************************************************
;*
;* ReadCluster
;*
;***************************************************************************
;*
;* Read CUR_CLUSTER_LO - CUR_CLUSTER_HI from SRAM assuming already set up
;*
;***************************************************************************/

ReadCluster		movlw		d'3'
			movwf		CTR
fatreadlp		bsf		PORTA,ENABLE
			movf		PORTD,0
			bcf		PORTA,ENABLE
			movwf		INDF
			incf		FSR,1
			incf		PORTC,1
			btfsc		STATUS,Z
			incf		PORTB,1
			decfsz		CTR,1
			goto		fatreadlp
			return

;***************************************************************************
;*
;* IdeProcessBuffers
;*
;***************************************************************************
;*
;* If there's space, read 8K of song data from CUR_CLUSTER into SRAM
;* at SRAM_WRITE. Once transfer is complete, update CUR_CLUSTER with the
;* next required cluster address, fetching a new FAT sector if required.
;*
;***************************************************************************/

IdeProcessBuffers
			; see if we have <=8K in SRAM
			movf		SRAM_COUNT_HI,0
			sublw		h'1f'		; 8K=0x2000
			btfss		STATUS,C	; C->+ve/zero,ct<0x20
			return				; NC -> return

			; ensure we haven't read the last data
			btfsc		CUR_STATUS,CS_LASTDATA
			return

			; we need to read the current cluster into SRAM
			movlw		SECTORS_PER_CLUSTER
			movwf		SEC_CT		; set count

			; convert current cluster to sector address
			call		ClusterToSector

			; set SRAM ptr
			movf		SRAM_WRITE_HI,0
			andlw		b'01111111'
			iorlw		b'01000000'
			movwf		SRAM_HI
			movf		SRAM_WRITE_LO,0
			movwf		SRAM_LO

			; read them sectors...
			call		IdeReadSectors

			; update SRAM count and pointer
			bsf		SRAM_COUNT_HI,5		; +8192
			movlw		h'20'			; $2000=8K
			xorwf		SRAM_WRITE_HI,1

			; figure out which FAT sector we need (CUR_CLUSTER/128)
			rlf		CUR_CLUSTER_LO,0
			rlf		CUR_CLUSTER_MID,0
			movwf		TMP0
			rlf		CUR_CLUSTER_HI,0
			movwf		TMP1

			; is this the sector we already have?
			xorwf		CUR_FAT_CACHED_HI,0
			btfss		STATUS,Z
			goto		fetch_fat
			movf		TMP0,0
			xorwf		CUR_FAT_CACHED_LO,0
			btfss		STATUS,Z
			goto		fetch_fat



already_got_fat		; we have the correct fat sector - read next cluster
			call		SetIOSRAMRead
			rlf		CUR_CLUSTER_LO,0
			movwf		PORTC
			rlf		PORTC,1
			rlf		PORTB,1
			movlw		b'00000001'
			andwf		PORTB,1
			movlw		b'11111100'
			andwf		PORTC,1

			; read data a byte at a time (little endian)
			movlw		CUR_CLUSTER_LO
			movwf		FSR
			call		ReadCluster

			; did we hit the next index cluster?
			movf		CUR_CLUSTER_LO,0
			xorwf		NEXT_CLUSTER_LO,0
			btfss		STATUS,Z
			goto		not_next
			movf		CUR_CLUSTER_MID,0
			xorwf		NEXT_CLUSTER_MID,0
			btfss		STATUS,Z
			goto		not_next
			movf		CUR_CLUSTER_HI,0
			xorwf		NEXT_CLUSTER_HI,0
			btfss		STATUS,Z
			goto		not_next

			; ok... we need to fetch the next index block
			incf		CUR_INDEX,1
			call		TrackReadIndex


not_next		; did we get ??FFFF??h ?
			incfsz		CUR_CLUSTER_MID,0
			return
			incfsz		CUR_CLUSTER_HI,0
			return

			; hit the end of the chain
			bsf		CUR_STATUS,CS_LASTDATA	; set flag

			; we just added 8K to SRAM_COUNT... but actually, we
			; want the audio to terminate at the last byte of
			; the tune... so we should actually add (LENGTH % 8192)
			; fetch the length from the track header...
			call		SetIOSRAMRead
			movlw		h'04'		; header at $0400
			movwf		PORTB
			movlw           TRKHDROFFSET_LEN
			movwf		PORTC
			bsf		PORTA,ENABLE
			movf		PORTD,0		; get LO byte
			andlw		b'11111100'	; ensure divisible by 4
			movwf		TMP0
			bcf		PORTA,ENABLE
			incf		PORTC,1		; move on
			bsf		PORTA,ENABLE
			movf		PORTD,0		; get HI byte
			andlw           h'1f'		; %8192 = &8191 = &1fff
			movwf		TMP1
			bcf		PORTA,ENABLE

			; only error case is if the track is divisible by 8192K
			; in which case, we added the correct amount!
			movf		TMP1,0
			btfsc		STATUS,Z
			return

			; subtract the 8K we added
			bcf		SRAM_COUNT_HI,5

			; add on the correct number of bytes
			movf		TMP0,0
			addwf		SRAM_COUNT_LO,1
			btfsc		STATUS,C
			incf		SRAM_COUNT_HI,1
			movf		TMP1,0
			addwf		SRAM_COUNT_HI,1

			; we're done
			return

fetch_fat		; ok... need to read the fat sector TMP1:TMP0
			movf		TMP1,0
			movwf		CUR_FAT_CACHED_HI
			movf		TMP0,0
			movwf		CUR_FAT_CACHED_LO

			; add TMP1:TMP0 to FAT_START, put into IDE_SEC
			addwf		FAT_START_0,0
			movwf		IDE_SEC0
			movf		TMP1,0
			movwf		IDE_SEC1
			movf		FAT_START_1,0
			btfsc		STATUS,C
			incfsz		FAT_START_1,0
			addwf		IDE_SEC1,1
			movf		FAT_START_2,0
			movwf		IDE_SEC2
			movlw		d'0'
			btfsc		STATUS,C
			movlw		d'1'
			addwf		IDE_SEC2,1
			btfsc		STATUS,C
			incf		IDE_SEC3,1

			; set SRAM ptr
			clrf		SRAM_LO
			clrf		SRAM_HI

			; we want 1 sector
			movlw		d'1'
			movwf		SEC_CT

			; read
			call		IdeReadSectors

			; continue as if we had the correct FAT all along
			goto		already_got_fat

;***************************************************************************
;*
;* LCDInit
;*
;***************************************************************************
;*
;* Initialise LCD
;*
;***************************************************************************/

LCDSlowInst		movwf		PORTC
			bsf		PORTA,ENABLE
			nop
			nop
			bcf		PORTA,ENABLE
			call		BigDelay
			return

LCDInit                 call		SetIOLCDWrite	; write inst by default
			call		BigDelay
			movlw		h'30'
			call		LCDSlowInst
			movlw		h'30'
			call		LCDSlowInst
			movlw		h'30'
			call		LCDSlowInst
			movlw		h'38'
			call		LCDSlowInst
			movlw		h'08'
			call		LCDSlowInst
			movlw		h'01'
			call		LCDSlowInst
			movlw		h'06'
			call		LCDSlowInst
			movlw		h'0c'
			call		LCDSlowInst
			return

;***************************************************************************/
;*
;* ModePaintTop
;*
;***************************************************************************
;*
;* paint the top row of the LCD display according to mode
;*
;***************************************************************************

ModePaintTop		movf		CUR_MODE,0
			call		StringOffsetTable
			movwf		TMP0

			movf		CUR_MODE,0
			call		CursorPosTable
			movwf		LCD_CURSOR

			; set SRAM write mode, address
			call		SetIOSRAMWrite
			movlw           d'20'
			movwf		CTR
			clrw
			call		LCDClearRow
			clrf		PORTC

			; set ctr
			movlw		d'16'
			movwf		CTR

			; copy message
setmodelcd_lp           movf		TMP0,0
			call		ModeMsgTable
			call		PrintChar
			incf		TMP0,1
			decfsz          CTR,1
			goto		setmodelcd_lp
lcd_eom

			; check '<' or '>' flags
			movf		CUR_MODE,0
			call            FlagTable
			movwf		TMP0
			btfss		TMP0,0
			goto		no_right_arrow

			decf		PORTC,1
			movlw		'>'
			movwf		PORTD
			bsf		PORTA,ENABLE
			bcf		PORTA,ENABLE

no_right_arrow          btfss		TMP0,1
			goto		no_left_arrow

			clrf		PORTC
			movlw		'<'
			movwf		PORTD
			bsf		PORTA,ENABLE
			bcf		PORTA,ENABLE

no_left_arrow
			return


;***************************************************************************/
;*
;* ModePaint
;*
;***************************************************************************
;*
;*
;*
;***************************************************************************

ModePaint		call		ModePaintTop
			call		ModePaintBottom
			return

;***************************************************************************/
;*
;* ModePaintBottom functions....
;*
;***************************************************************************
;*
;*
;*
;***************************************************************************

mpb_volume		bsf		STATUS,RP0
			rrf		CUR_VOLUME,0
			bcf		STATUS,RP0
			call		BlockPrint
			return

mpb_treble		bsf		STATUS,RP0
			rrf		CUR_TREBLE,0
			bcf		STATUS,RP0
			call		BlockPrint
			return

mpb_bass		bsf		STATUS,RP0
			rrf		CUR_BASS,0
			bcf		STATUS,RP0
			call		BlockPrint
			return

			; write CD title and ID
mpb_cd			movlw           h'44'
			movwf		TMP0
			movlw		d'12'
			movwf		CTR
			movlw           (SRAM_ADDR_CDHDR>>8)
			movwf		SRAM_HI
			movlw           CDHDROFFSET_TITLE
			movwf		SRAM_LO
			call		LCDMessage
			movlw		h'40'
			movwf		PORTC
			movf            CUR_CDHI,0
			movwf		TMP1
			incf            CUR_CDLO,0
			movwf		TMP0
			btfsc		STATUS,Z
			incf		TMP1,1
			call            DecPrint
			incf		PORTC,1
			movlw		'-'
			call		PrintChar
			return

mpb_trk			movlw           h'44'
			movwf		TMP0
			movlw		d'12'
			movwf		CTR
			movlw           (SRAM_ADDR_TRKHDR>>8)
			movwf		SRAM_HI
			movlw           TRKHDROFFSET_TITLE
			movwf		SRAM_LO
			call		LCDMessage
			movlw		h'40'
			movwf		PORTC
			incf            CUR_TRACK,0
			movwf		TMP0
			clrf		TMP1
			call            DecPrint
			movlw		'-'
			incf		PORTC,1
			call		PrintChar
			return



;***************************************************************************
;*
;* LCDClearRow
;*
;***************************************************************************
;*
;* Clear row in SRAM pointed to by W, assume CTR set up
;*
;***************************************************************************/

LCDClearAll		call		SetIOSRAMWrite
			clrw
			clrf		CTR
LCDClearRow             movwf		PORTC
			movlw		(SRAM_ADDR_LCD>>8)
			movwf		PORTB
			movlw		d'32'
			movwf		PORTD
lcd_clear_lp		bsf		PORTA,ENABLE
			bcf		PORTA,ENABLE
			incf		PORTC,1
			decfsz		CTR,1
			goto		lcd_clear_lp
			return

;***************************************************************************
;*
;* LCDMessage
;*
;***************************************************************************
;*
;* TMP0 = lcd screen buffer offset
;* CTR  = max chars to copy
;* SRAM_HI:SRAM_LO = source in SRAM (START OF MESSAGE!!!)
;*
;***************************************************************************/

LCDMessage
			; set scroll offset
			movf		SCROLL_OFFSET,0
			btfsc		SCROLL_OFFSET,7
			clrw
			movwf           TMP2

			; clear area first
			movf		CTR,0
			movwf		TMP1
			movf		TMP0,0
			call		LCDClearRow

			; read message into TRANS_BUFFER+1
			call		SetIOSRAMRead
			movf		SRAM_HI,0
			movwf		PORTB
			movf		SRAM_LO,0
			addwf		TMP2,0
			movwf		PORTC
			movlw		TRANS_BUFFER+1
			movwf		FSR
			movf		TMP1,0		; retrieve counter
			movwf		CTR

lcd_read_lp             bsf		PORTA,ENABLE
			movf		PORTD,0
			bcf		PORTA,ENABLE
			movwf           INDF
			btfss		STATUS,Z
			goto		non_zero

			; got zero terminating char... if no offset, msg fits ok
			movf		TMP2,0
			btfsc		STATUS,Z
			goto		msg_fits

			; else, write gap symbol, reset src
			movlw		':'
			movwf		INDF
			movf		SRAM_LO,0
			movwf		PORTC
			decf		PORTC,1		; precomp

			; if this was the first character, reset scroll offset!
			movf            CTR,0
			xorwf		TMP1,0
			btfsc		STATUS,Z
			bsf             LCD_STATUS,LCD_MSG_SOZ

non_zero		incf		FSR,1
			incf            PORTC,1
			decfsz		CTR,1
			goto		lcd_read_lp

			; write to LCD
lcd_write		call		SetIOSRAMWrite
			movlw           (SRAM_ADDR_LCD>>8)
			movwf		PORTB
			movf            TMP0,0
			movwf		PORTC
			movlw		TRANS_BUFFER+1
			movwf		FSR

lcd_write_lp		decf		TMP1,1
			btfsc		TMP1,7
			goto		lcd_write_done
			movf		INDF,0
			movwf		PORTD
			bsf		PORTA,ENABLE
			bcf		PORTA,ENABLE
			incf		FSR,1
			incf		PORTC,1
			goto		lcd_write_lp

lcd_write_done		return

			; keep scroll offset negative
msg_fits		bsf             LCD_STATUS,LCD_MSG_SOZ
			movf		CTR,0
			subwf           TMP1,1
			goto		lcd_write

;***************************************************************************
;*
;*
;*
;***************************************************************************
;*
;*
;*
;***************************************************************************/

UpdateTime              bcf		LCD_STATUS,LCD_TIMEUPDATE
			call		ModePaintBottom
			return

;***************************************************************************
;*
;*
;*
;***************************************************************************
;*
;*
;*
;***************************************************************************/

ScrollMessage
			; clear flag
			bcf		LCD_STATUS,LCD_SCROLL

			; scroll it
			incf            SCROLL_OFFSET,1

			; check if it needs resetting?
			btfsc		LCD_STATUS,LCD_MSG_SOZ
			clrf		SCROLL_OFFSET
			bcf		LCD_STATUS,LCD_MSG_SOZ

			; update msg
			call		ModePaintBottom
			return

;***************************************************************************
;*
;* DecPrint
;*
;***************************************************************************
;*
;* Write TMP1:TMP0 as a 3-digit decimal number, assume PORTB, PORTC already
;* set up...
;*
;***************************************************************************/

DecPrint                movlw		d'2'
			movwf		CTR

			; divide by column value
col_lp                  decf		CTR,0
			call            DecGetLo
			movwf           TRANS_BUFFER

			; init counter
			movlw		-d'1'
			movwf           TMP2

			; do repeated subtraction (addition)
sublp			incf		TMP2,1

			; subtract TRANS_BUFFER from TMP1:TMP0
			movf		TRANS_BUFFER,0
			subwf           TMP0,1
			btfss		STATUS,C
			decf		TMP1,1
			btfss		TMP1,7
			goto		sublp

			; restore last value
			movf		TRANS_BUFFER,0
			addwf		TMP0,1
			btfsc		STATUS,C
			incf		TMP1,1

			; add ascii val
			movlw           '0'
			addwf           TMP2,0
			movwf		PORTD
			bsf		PORTA,ENABLE
			bcf		PORTA,ENABLE
			incf		PORTC,1

			decfsz		CTR,1
			goto		col_lp

			; final digit should be in TMP0
			movlw           '0'
			addwf           TMP0,0
			movwf		PORTD
			bsf		PORTA,ENABLE
			bcf		PORTA,ENABLE
			return

;***************************************************************************
;*
;* BlockPrint
;*
;***************************************************************************
;*
;* Write w/2 blocks on the bottom row of the LCD, for volume/tone etc.
;*
;***************************************************************************/

BlockPrint              movwf		TMP0
			rrf		TMP0,1
			movlw           d'32'
			movwf		CTR
			movlw           h'40'
			call		LCDClearRow

			movlw		h'43'
			movwf		PORTC
			movlw		'['
			call		PrintChar
			movlw		h'4b'
			movwf		PORTC
			movlw		']'
			call		PrintChar

			movf		TMP0,0
			btfsc           STATUS,Z
			return

			movlw		d'7'
			andwf		TMP0,1
			btfsc		STATUS,Z
			return
			movlw           h'44'
			movwf           PORTC
			movlw		h'ff'
			movwf		PORTD

block_lp		bsf		PORTA,ENABLE
			bcf		PORTA,ENABLE
			incf		PORTC,1
			decfsz		TMP0,1
			goto		block_lp
			return

;***************************************************************************
;*
;*
;*
;***************************************************************************
;*
;*
;*
;***************************************************************************/

GetTrackCount		; read from SRAM
			call		SetIOSRAMRead
			movlw		(SRAM_ADDR_CDHDR>>8)
			movwf		PORTB
			movlw           CDHDROFFSET_TRKCOUNT
			movwf		PORTC
			bsf		PORTA,ENABLE
			movf		PORTD,0
			bcf		PORTA,ENABLE
			return

;***************************************************************************
;*
;*
;*
;***************************************************************************
;*
;*
;*
;***************************************************************************/

GetCdCount		; read from SRAM
			call		SetIOSRAMRead
			movlw		(SRAM_ADDR_FSHDR>>8)
			movwf		PORTB
			movlw           HDROFFSET_CDCOUNT
			movwf		PORTC
			bsf		PORTA,ENABLE
			movf		PORTD,0
			movwf           TMP0
			bcf		PORTA,ENABLE
			incf		PORTC,1
			bsf		PORTA,ENABLE
			movf		PORTD,0
			movwf           TMP1
			bcf		PORTA,ENABLE
			return

;***************************************************************************
;*
;*
;*
;***************************************************************************
;*
;*
;*
;***************************************************************************/

TrackFinished		; move onto next track!
			bcf             CUR_STATUS,CS_TUNEDONE
			call		TrackStop
			movlw		MODE_TRK
			movwf		CUR_MODE
			movlw		VB_TRK_PLUS
			movwf		PROC_BUTTON
			bsf		CUR_STATUS,CS_STREAMOK
			goto		ButtonHandler

;***************************************************************************
;*
;* Delay<n>
;*
;***************************************************************************
;*
;* Handy quick delay functions
;*
;***************************************************************************/

Delay6			nop
Delay5			nop
Delay4			return


;***************************************************************************
;*
;* I2CWaitForClock
;*
;***************************************************************************
;*
;* Wait for clock to go high (ensure slave isn't stalling us)
;* Assumes we're in page 1, and leaves us that way
;*
;***************************************************************************/

I2CWaitForClock		bcf		STATUS,RP0		; access ports
i2c_wait		btfss		PORTA,I2CC		; wait for clock hi
			goto		i2c_wait
			bsf		STATUS,RP0		; go to page 1
			return


;***************************************************************************
;*
;* I2CStart
;*
;***************************************************************************
;*
;* Assumes both lines are high - send a start condition
;* Assumes we're in page 1, and leaves us that way
;*
;***************************************************************************/

I2CStart                call		I2CWaitForClock
			bcf		TRISA,I2CD		; lower data
			call		Delay4			; wait
			return					; that's all


;***************************************************************************
;*
;* I2CByte
;*
;***************************************************************************
;*
;* Assumes I2CC is hi - send the byte in I2C_BYTE, store ack in I2C_RESPONSE
;* Assumes we're in page 1, and leaves us that way
;* Leave I2CC high (middle of ACK)
;*
;***************************************************************************/

I2CByte			movlw		d'8'			; set ctr
			movwf		I2C_CTR

			; send 1 bit at a time
I2CBit                  call		I2CWaitForClock		; wait for clock

			bcf		TRISA,I2CC		; lower clock
			nop

			rlf		I2C_BYTE,1		; shift
			bcf		TRISA,I2CD		; assume low
			btfsc           STATUS,C		; test carry
			bsf		TRISA,I2CD		; set data

			nop					; data settle

			bsf		TRISA,I2CC		; raise clock
			decfsz		I2C_CTR,1		; keep going
			goto		I2CBit

			; test ack
			call		Delay6
			call		I2CWaitForClock		; wait for clock
			bcf		TRISA,I2CC		; lower clock

			bsf		TRISA,I2CD		; release data
			call            Delay4			; delay

			bsf             TRISA,I2CC		; raise clock
			call		I2CWaitForClock		; ensure clock is actually high

			bcf		STATUS,RP0              ; go to page 0
			movf		PORTA,0			; read port
			bsf		STATUS,RP0		; go to page 1

			movwf		I2C_RESPONSE		; store port value

			return					; done

;***************************************************************************
;*
;* I2CStop
;*
;***************************************************************************
;*
;* Assumes clock is high (middle of ack) - send a stop condition
;* Assumes we're in page 1, and leaves us that way
;*
;***************************************************************************/

I2CStop			call		I2CWaitForClock		; wait for clock
			bcf		TRISA,I2CC		; lower clock clock
			call		Delay4			; wait
			bcf		TRISA,I2CD		; lower data (should be low anyway!)
			call		Delay4			; wait
			bsf		TRISA,I2CC		; raise clock
			call		Delay4			; wait
			bsf             TRISA,I2CD		; raise data
			return					; that's all

;***************************************************************************
;*
;* I2CProcess
;*
;***************************************************************************
;*
;* See if we need to process any I2C data - send a byte at a time if so...
;*
;***************************************************************************/

I2CProcess              btfss		LCD_STATUS,LCD_I2CBUSY	; return if not busy
			return

			movf		PORTA,0			; ensure both low
			andlw		b'11111100'		; when outputs
			movwf		PORTA			; are enabled

			bsf		STATUS,RP0		; go to page 1

			btfsc		I2C_MSGLEN,7		; start req?
			call            I2CStart
			bcf		I2C_MSGLEN,7		; clear bit

			movf		I2C_MSGPOS,0		; get byte offset
			addlw           I2C_MSG			; get address
			movwf		FSR			; put in FSR
			movf		INDF,0			; get byte
			movwf           I2C_BYTE		; store it
			call            I2CByte			; send byte
			btfsc		I2C_RESPONSE,I2CD	; test resp. bit
			goto            i2c_done		; no ack?

			incf		I2C_MSGPOS,1		; inc ptr
			movf		I2C_MSGPOS,0		; get ptr
			xorwf           I2C_MSGLEN,0		; test end?
			btfsc		STATUS,Z
			goto            i2c_done		; if so, stop
			bcf		STATUS,RP0		; go to page 0
			return					; we're done

i2c_done                call		I2CStop			; send stop signal
			bcf		STATUS,RP0		; go to page 0
			bcf             LCD_STATUS,LCD_I2CBUSY	; clear busy bit
			return					; we're done


;***************************************************************************
;*
;*
;*
;***************************************************************************/

PrintChar		movwf           PORTD
			bsf		PORTA,ENABLE
			bcf		PORTA,ENABLE
			incf		PORTC,1
			return

;***************************************************************************
;*
;*
;*
;***************************************************************************/

ShowTime                ; assume portB,C set up already
			movlw           d'32'
			movwf		CTR
			movlw           h'40'
			call		LCDClearRow

			movlw		h'40'
			movwf		PORTC

			movlw		'['
			call		PrintChar
			clrf		TMP1
			bsf		STATUS,RP0
			movf            TIME_MIN,0
			bcf		STATUS,RP0
			andlw		b'00111111'
			movwf           TMP0
			movlw		d'1'
			movwf		CTR
			call		col_lp
			incf		PORTC,1
			movlw		':'
			call		PrintChar
			clrf		TMP1
			bsf		STATUS,RP0
			movf            TIME_SEC,0
			bcf		STATUS,RP0
			movwf           TMP0
			movlw		d'1'
			movwf		CTR
			call		col_lp
			incf		PORTC,1
			movlw		']'
			call		PrintChar

			movlw           h'47'
			movwf		TMP0
			movlw		d'9'
			movwf		CTR
			movlw           (SRAM_ADDR_TRKHDR>>8)
			movwf		SRAM_HI
			movlw           TRKHDROFFSET_TITLE
			movwf		SRAM_LO
			call		LCDMessage

			return


;***************************************************************************
;*
;* AdjustParm
;*
;***************************************************************************
;*
;* Adjust a 0-15 range variable
;*
;***************************************************************************/

AdjustParm		movlw		d'1'
			btfsc		PROC_BUTTON,0
			movlw		-d'1'
			addwf		INDF,0
			movwf		TMP0
			btfss		TMP0,5
			movwf		INDF
			return

;***************************************************************************
;*
;* I2CTreble
;*
;***************************************************************************
;*
;*
;*
;***************************************************************************/

I2CTreble		bsf		STATUS,RP0
			movf		CUR_TREBLE,0
			movwf           TMP_PAGE1
			movlw           h'f0'
			movwf		I2C_MSG+3
			goto            I2CTBContinue

I2CBass			bsf		STATUS,RP0
			movf		CUR_BASS,0
			movwf           TMP_PAGE1
			movlw           h'b0'
			movwf		I2C_MSG+3

I2CTBContinue
			rlf		TMP_PAGE1,1	; value*2 (00-40)
			rlf		TMP_PAGE1,1	; value*4 (00-80)
			movlw		b'11111100'
			andwf		TMP_PAGE1,0	; 00-80
			addlw		-h'40'

			movwf		I2C_MSG+4
			clrf		I2C_MSG+5

			movlw		d'6' | d'128'
			movwf		I2C_MSGLEN
			clrf		I2C_MSGPOS
			movlw		h'3a'
			movwf           I2C_MSG
			movlw		h'68'
			movwf           I2C_MSG+1
			movlw		h'96'
			movwf           I2C_MSG+2
			bcf		STATUS,RP0
			bsf		LCD_STATUS,LCD_I2CBUSY
			return

I2CVolume		bsf		STATUS,RP0
			movlw           d'4' | d'128'
			movwf		I2C_MSGLEN
			clrf		I2C_MSGPOS
			movlw		h'9a'
			movwf		I2C_MSG
			movlw		h'c2'
			movwf		I2C_MSG+1
			rlf		CUR_VOLUME,0
			movwf		TMP_PAGE1
			movwf           I2C_MSG+2
			movwf		I2C_MSG+3
			bcf		STATUS,RP0
			bsf		LCD_STATUS,LCD_I2CBUSY
			return

;***************************************************************************
;*
;*
;*
;***************************************************************************
;*
;*
;*
;***************************************************************************/

I2CFlush		btfss		LCD_STATUS,LCD_I2CBUSY
			return
			call            I2CProcess
			goto		I2CFlush

;***************************************************************************
;*
;*
;*
;***************************************************************************
;*
;* check for the existance of the 4-letter string at FSR at PORTB:PORTC
;* in SRAM... in case of error, go to LcdError
;*
;***************************************************************************/

VerifyChar		bsf		PORTA,ENABLE
			xorwf		PORTD,0
			bcf		PORTA,ENABLE
			btfss		STATUS,Z
			goto		VerifyError
			incf		PORTC,1
			return

VerifyMP                movlw		'M'
			call		VerifyChar
			movlw		'P'
			call		VerifyChar
			return

VerifyStart             movwf		PORTB
			clrf		PORTC
			call		VerifyMP
			return

VerifyTrk		call		SetIOSRAMRead
			movlw           (SRAM_ADDR_TRKHDR>>8)
			call		VerifyStart
			movlw		'T'
			call            VerifyChar
			movlw		'K'
			call		VerifyChar
			return

VerifyCd		call		SetIOSRAMRead
			movlw           (SRAM_ADDR_CDHDR>>8)
			call		VerifyStart
			movlw		'C'
			call            VerifyChar
			movlw		'D'
			call		VerifyChar
			return

VerifyFs		call		SetIOSRAMRead
			movlw           (SRAM_ADDR_FSHDR>>8)
			call		VerifyStart
			movlw		'F'
			call            VerifyChar
			movlw		'S'
			call		VerifyChar
			return

VerifyError             call		LCDClearAll
			movlw		d'15'
			movwf		CTR
			clrf		PORTC
err_lp                  decf		CTR,0
			call            ErrorMsgTable
			call		PrintChar
			decfsz		CTR,1
			goto		err_lp
vhalt			call		LCDProcess
			call		TimerProcess
			goto		vhalt


			end