;*******************************************************************************
;                       Mp3 PuBliC                                             *
;                                                                              *
; Project Members : Ben Stragnell                                              *
;                   Leandro Gentili                                            *
;                   Michael Weston                                             *
;                                                                              *
; Firmware Ver 1.1.0                                                           *
; - reference this number to make sure you are using the most current version  *
;*******************************************************************************




	      org 0x0800
;*********************************************************************
;*  Initial start up code after boot code passes execution to firmware
;*********************************************************************
FIRMWARE_START
		; Part of MP3-HD code must be repeated here due to timing issues of PC software
		; Parallel port must be checked before ATAPI test.  This was the cleanest way to do it
		; without bouncing back and forth in code too much.  These registers are cleared in case PC
		; debug mode is to be started.
		movlw		h'7f' 
		movwf		FSR
clrlp2		clrf		INDF
		bsf		FSR,7		
		clrf		INDF
		bcf		FSR,7
		decf		FSR,1
		movlw		h'1f'
		xorwf           FSR,0
		btfss		STATUS,Z
		goto		clrlp2
		movlw		d'1'
		movwf		SCROLL_TIMER
		movlw		0x10
		movwf		PCLATH

		call            LCDInit

		; test for 'AA' on latch - PC debug signal
		call		SetIOLatchReadWrite
		bsf		PORTA,ENABLE
		movf		PORTD,0
		bcf		PORTA,ENABLE
		xorlw		h'AA'
		btfsc		STATUS,Z
		goto		PCChat
		movlw		0x08
		movwf		PCLATH

#ifdef F10_OR_HIGHER			; This code is not necessary on D8 chips
		movlw .6		;NOTE- MPEG data is 20 bits, frame is 32 bits
              	movwf N_BYTEIIC		;send I2C commands to clear bits 0...11
              	movlw 0x3A		;of I2S output that are not used.  By doing
              	movwf BUFFERIIC		;this, pops and clicks will be eliminated
              	movlw 0x68		;when an MPEG frame synchronizes. 
              	movwf BUFFERIIC+1	
              	movlw 0x9C
              	movwf BUFFERIIC+2
          	movlw 0x50
          	movwf BUFFERIIC+3
		clrf BUFFERIIC+4
		clrf BUFFERIIC+5
		call SEND_BUFFER
		movlw .6
              	movwf N_BYTEIIC
		movlw 0x60
          	movwf BUFFERIIC+3
		call SEND_BUFFER
#endif
		
		;goto DONE_INITIALIZING
		
#ifndef XTAL_14.725MHz			; If not using 14.725MHz, we must change the PLL values
		call INI_IIC 		; This first I2C command is too long to call SEND_BUFFER
		call START_IIC
		movlw .16
		movwf I2C_COUNT
PLL_LOOP
		movf I2C_COUNT,0
		call PLL_TABLE
		movwf BYTEIIC
 		call OUT_IIC
		call GETACK_IIC
		btfsc I2C_FAIL,0
		goto I2C_CMND_FAILED
		decfsz I2C_COUNT,1
		goto	PLL_LOOP
		call STOP_IIC

		movlw .4		; Now send RUN command
              	movwf N_BYTEIIC	
              	movlw 0x3A
              	movwf BUFFERIIC	
              	movlw 0x68
              	movwf BUFFERIIC+1
	#ifdef F10_OR_HIGHER
              	movlw 0x04
              	movwf BUFFERIIC+2
          	movlw 0x75
          	movwf BUFFERIIC+3
	#else
		movlw 0x0F		; Different address for D8 version
              	movwf BUFFERIIC+2
          	movlw 0xCD
          	movwf BUFFERIIC+3
	#endif
		call SEND_BUFFER
		goto I2C_CMND_SUCCESSFUL
I2C_CMND_FAILED
		call STOP_IIC
#endif

I2C_CMND_SUCCESSFUL
;************************************************************
;* The following code is only used with G10, G12 versions
;* of the MAS3507D chip. Do not define if you don't have
;* this chip.
;************************************************************
#ifdef G10_STARTUP
		movlw .6
              	movwf N_BYTEIIC	
              	movlw 0x3A
              	movwf BUFFERIIC	
              	movlw 0x68
              	movwf BUFFERIIC+1
              	movlw 0x93
              	movwf BUFFERIIC+2
          	movlw 0xB0
          	movwf BUFFERIIC+3
		clrf BUFFEIIC+4
		movlw 0x02
		movwf BUFFEIIC+5
		call SEND_BUFFER
		movlw .4
              	movwf N_BYTEIIC	
              	clrf BUFFERIIC+2
          	movlw 0x01
          	movwf BUFFERIIC+3
		call SEND_BUFFER
#endif
DONE_INITIALIZING
		movlw 0x44
             	movwf BUFFERIIC+2
              	call INI_DAC
		movlw	0x18		; Go to CD code which will check for an ATAPI device
		movwf	PCLATH
		goto	START_CD


;************************************************************
;* The following code is used by the MP3-CD code
;************************************************************
CLEAR_D
              movlw 0x01
	      goto COMANDO
RETURN_H 
              movlw 0x02
	      goto COMANDO
MODE_SET	
              movlw 0x04
              goto COMANDO	
DISPLAY_C	
              movlw 0x08
	      goto COMANDO
SHIFT	        
              movlw 0x10
              goto COMANDO
FUNC_SET  	
              movlw 0x20
	      goto COMANDO
SET_CG_AD       
              movlw 0x40 
	      goto COMANDO
SET_DD_AD	 
              movlw 0x80

;**********************************************************
COMANDO
              bsf STATUS,RP0
              clrf TRISC
              bcf STATUS,RP0 
    
              iorwf REG_LCD,w
              movwf PORTC
	      bcf PORTB,RSn
	      bcf PORTB,RWn                  

	      goto SEND2LCD
;**********************************************************
;**********************************************************
WRITE_DATA        
              bsf STATUS,RP0
              clrf TRISC
              bcf STATUS,RP0
                  
              movwf PORTC
              bsf PORTB,RSn
              bcf PORTB,RWn
;**********************************************************
;**********************************************************
SEND2LCD
              nop   
   	      movlw E_LCD
              movwf PORTE   
              bsf PORTA,HAB_DEC
              nop
              nop    
	      bcf PORTA,HAB_DEC
	      return
;**********************************************************
;**********************************************************                               

MENSAJE_L1      
              clrf REG_LCD
              call SET_DD_AD
              call ESPERA3    
              goto SALTA
MENSAJE_L2
              movlw 0x40
              movwf REG_LCD
              call SET_DD_AD
              call ESPERA3
SALTA
              clrf COUNTER
DALENOMA                  
              movf COUNTER,w 
              call MANDA
              call WRITE_DATA
              call ESPERA3
              incf COUNTER 
              movf COUNTER,w
              sublw .16
              btfss STATUS,Z
              goto DALENOMA
              clrf MESSAJE
              clrf MESSAJE1 
              return
MANDA                   
              btfsc MESSAJE,0
              addlw M_0-BASE2
              btfsc FLAG_CD,4
              addlw M_1-BASE2
              btfsc MESSAJE,2
              addlw M_2-BASE2
              btfsc MESSAJE,3
              addlw M_3-BASE2
              btfsc MESSAJE,4
              addlw M_4-BASE2
              btfsc MESSAJE,5
              addlw M_5-BASE2
              btfsc MESSAJE,6
              addlw M_6-BASE2
              btfsc MESSAJE,7
              addlw M_7-BASE2     
              btfsc MESSAJE1,0
              addlw M_8-BASE2
              btfsc MESSAJE1,1
              addlw M_9-BASE2 
              btfsc FLAG,ATAPIn
              addlw M_10-BASE2  
              btfsc MESSAJE1,2
              addlw M_11-BASE2
MORO                 
              movwf TEMPW
              movlw 0x0F
              movwf PCLATH  
              movf TEMPW,w
              movwf PCL
;**********************************************************
;**********************************************************                    
NO_CURSOR_L2
              movlw 0x40
              movwf REG_LCD
              call SET_DD_AD
              call ESPERA3

              movlw b'00000100'
              movwf REG_LCD
              call DISPLAY_C
              call ESPERA3
              return
;**********************************************************
;**********************************************************
CURSOR_L2         
              movlw 0x40
              movwf REG_LCD
              call SET_DD_AD
              call ESPERA3
                 
              movlw b'00000111'
              movwf REG_LCD
              call DISPLAY_C
              call ESPERA3
              return
;**********************************************************
;**********************************************************
NAME              
              movf ADR_RL,w
              movwf BP0
              movf ADR_RH,w
              movwf BP1
              movf ADR_WL,w
              movwf BP2
              movf ADR_WH,w
              movwf BP3

              clrf REG_LCD
              call SET_DD_AD
              call ESPERA3

              movlw 0x18
	      movwf PCLATH
              call CAL_BASE
              movlw .8
              movwf PCLATH
              movlw .2
              movwf COUNTER1

              btfsc FLAG_CD,5
              goto T_CDROM  
              movlw 0x18
	      movwf PCLATH
              call CAL_DEST_DA 
              movlw .8
              movwf PCLATH
              goto ELA 
T_CDROM
              movlw 0x18
	      movwf PCLATH
              call CAL_DEST_CDROM
              movlw .8
              movwf PCLATH
ELA                  
              movlw .16
              movwf COUNTER
KAKA                
              movlw 0x18
	      movwf PCLATH
              call RD_RAM                 
              movlw .8
              movwf PCLATH
              movf BYTERAM,w
              call WRITE_DATA
              call ESPERA3

              incfsz ADR_RL
              goto MONGO
              incf ADR_RH
MONGO
              decfsz COUNTER  
              goto KAKA
              decfsz COUNTER1
              goto LINEA2             

              btfsc FLAG_STATE,MODE
              goto SALTAMOS
              movlw 0x18
	      movwf PCLATH
              call CAL_BASE
              call RD_RAM
              movlw .8
              movwf PCLATH                                    

              movf BYTERAM,w
              sublw 0x00
              btfss STATUS,Z
              call CURSOR_L2
SALTAMOS
              movf BP0,w
              movwf ADR_RL 
              movf BP1,w
              movwf ADR_RH
              movf BP2,w
              movwf ADR_WL 
              movf BP3,w
              movwf ADR_WH
              return
LINEA2
              call NO_CURSOR_L2
              goto ELA                                
;**********************************************************
;**********************************************************
NUMERO_SONG
              bsf MESSAJE,5    
              call MENSAJE_L2                  
                  
              movlw 0x4D
              movwf REG_LCD
              call SET_DD_AD
              call ESPERA3

              movf TOTAL_SONG,w 
              call BCD2LCD

              call ESPERA2         
              call ESPERA2
              call ESPERA2
              call ESPERA2
              call ESPERA2
              call ESPERA2
              call ESPERA2
              call ESPERA2
              call ESPERA2
              call ESPERA2
              call ESPERA2
              call ESPERA2
              call ESPERA2
              call ESPERA2
              call ESPERA2
              call ESPERA2
              call ESPERA2
              call ESPERA2
              return
;**********************************************************
;**********************************************************
MOSTRAR_CONTROL
              btfsc FLAG_CD,5        
              goto LENTO 
              movlw .230
              movwf CONT2
              clrf CONT1
              goto SLENTO
LENTO
              movlw .150
              movwf CONT1
              movlw .253
              movwf CONT2
SLENTO
              btfsc FLAG_CD,5
              goto DOS2
              movf CONTROL,w
              sublw .2
              btfsc STATUS,Z
              goto TRACK_MENU
DOS2
              movf CONTROL,w           
              sublw .4
              btfsc STATUS,Z
              goto TRACK_MENU
      
              movf CONTROL,w
              sublw .1
              btfsc STATUS,Z
              goto VOLUMEN_MENU

              movf CONTROL,w
              sublw .2
              btfsc STATUS,Z
              goto BASS_MENU                      
 
              call T_TEXT
              return
VOLUMEN_MENU
              call V_TEXT
              return
BASS_MENU
              call B_TEXT
              return
TRACK_MENU
              clrf CONTROL
              call NAME
              bcf FLAG_STATE,MODE
              return     
;**********************************************************
;**********************************************************
V_TEXT
              bsf MESSAJE,6
              call MENSAJE_L1 
     
              bsf MESSAJE1,1
              call MENSAJE_L2

              movlw 0x46
              movwf REG_LCD
              call SET_DD_AD
              call ESPERA3

              movf VOLUMEN,w
              call BCD2LCD1

              return
;**********************************************************
;**********************************************************          
B_TEXT
              bsf MESSAJE,7
              call MENSAJE_L1 
               
              bsf MESSAJE1,1
              call MENSAJE_L2
              call ESPERA3

              movlw 0x45
              movwf REG_LCD
              call SET_DD_AD
              call ESPERA3

     
              movlw .8
              subwf BASS,w
              movwf TEMPW        
      
              btfss STATUS,C
              goto NEGA                   
           
              movlw .43
              call WRITE_DATA
              call ESPERA3 

              movf TEMPW,w

              call BCD2LCD1
              return
NEGA              
              movlw .45
              call WRITE_DATA
              call ESPERA3 

              comf TEMPW
              incf TEMPW,w

              call BCD2LCD1
              return
;**********************************************************
;**********************************************************          
T_TEXT
              bsf MESSAJE1,0
              call MENSAJE_L1 
               
              bsf MESSAJE1,1
              call MENSAJE_L2
              call ESPERA3

              movlw 0x45
              movwf REG_LCD
              call SET_DD_AD
              call ESPERA3
    
              movlw .8
              subwf TREBLE,w
              movwf TEMPW        
      
              btfss STATUS,C
              goto NEGA1                   
           
              movlw .43
              call WRITE_DATA
              call ESPERA3 

              movf TEMPW,w

              call BCD2LCD1
              return
NEGA1              
              movlw .45
              call WRITE_DATA
              call ESPERA3 

              comf TEMPW
              incf TEMPW,w

              call BCD2LCD1
              return
;**********************************************************
;**********************************************************
BCD2LCD
              movwf L_BYTE
              movlw 0x18
	      movwf PCLATH
              call BCD
              movlw .8
              movwf PCLATH

              movf DIG_3,w
              call WRITE_DATA
              call ESPERA3
              movf DIG_2,w
              call WRITE_DATA
              call ESPERA3 
              movf DIG_1,w
              call WRITE_DATA
              call ESPERA3

              return
;**********************************************************
;**********************************************************
BCD2LCD1
              movwf L_BYTE
              movlw 0x18
	      movwf PCLATH
              call BCD
              movlw .8
              movwf PCLATH

              movf DIG_2,w
              call WRITE_DATA
              call ESPERA3 
              movf DIG_1,w
              call WRITE_DATA
              call ESPERA3

              return

;**********************************************************
;        DeLaYs
;**********************************************************                          
ESPERA2   
              movlw .255
              movwf ACC1
              movwf ACC2
              goto L2
ESPERA3   
              movlw .1        
              movwf ACC2
              movlw .1
              movwf ACC2
L2                      
              nop                 ; Aprox. 1 MicroSec.
              nop
              decfsz ACC1
              goto L2
              decfsz ACC2
              goto L2
              return

CONTROLS
              movlw 0x18
	      movwf PCLATH
              call REW_PRESS              
              movlw .8
              movwf PCLATH
              btfsc FLAG_KEY1,REW
              goto RESTO_C

              movlw 0x18
	      movwf PCLATH
              call FRD_PRESS
              movlw .8
              movwf PCLATH
              btfsc FLAG_KEY1,FRD
              goto SUMO_C
              movlw 0x18
	      movwf PCLATH
              return
SUMO_C
              movf CONTROL,w
              sublw .1
              btfsc STATUS,Z
              goto VOLU

              movf CONTROL,w
              sublw .2
              btfsc STATUS,Z
              goto BAS             

              movf TREBLE,w
              sublw .16
              btfss STATUS,Z
              incf TREBLE
              call TREBLE_MAS                
              goto MOST
VOLU
              movf VOLUMEN,w
              sublw .15 
              btfss STATUS,Z
              incf VOLUMEN
              call VOLUMEN_DAC            
              goto MOST
BAS
              movf BASS,w
              sublw .16
              btfss STATUS,Z
              incf BASS
              call BASS_MAS   
              goto MOST 
;********************************************
RESTO_C
              movf CONTROL,w
              sublw .1
              btfsc STATUS,Z
              goto VOLU1

              movf CONTROL,w
              sublw .2
              btfsc STATUS,Z
              goto BAS1

              movf TREBLE,w
              btfss STATUS,Z 
              decf TREBLE
              call TREBLE_MAS                         
              goto MOST
VOLU1
              movf VOLUMEN,w
              btfss STATUS,Z
              decf VOLUMEN
              call VOLUMEN_DAC          
              goto MOST
BAS1
              movf BASS,w
              btfss STATUS,Z
              decf BASS
              call BASS_MAS              
MOST
              call MOSTRAR_CONTROL
              return                             
;**********************************************************
;**********************************************************
INI_DAC
              movlw .3
              movwf N_BYTEIIC
              movlw 0x9A
              movwf BUFFERIIC
              movlw 0xC3
              movwf BUFFERIIC+1

              call SEND_BUFFER

              return
;**********************************************************
;**********************************************************
VOLUMEN_DAC
              movlw .4
              movwf N_BYTEIIC
              movlw 0x9A
              movwf BUFFERIIC
              movlw 0xC2
              movwf BUFFERIIC+1
              
              movf VOLUMEN,w
              addlw VOLUMEN_TABLE-BASE1
              call MORO2

              movwf BUFFERIIC+2
              movwf BUFFERIIC+3

              call SEND_BUFFER

              return
;**********************************************************
;**********************************************************               
BASS_MAS
              movf BASS,w
              movwf PRE
              call PRE_MAS

              movlw .6
              movwf N_BYTEIIC
              movlw 0x3A
              movwf BUFFERIIC
              movlw 0x68
              movwf BUFFERIIC+1
              movlw 0x96
              movwf BUFFERIIC+2
              movlw 0xB0
              movwf BUFFERIIC+3
             
              bcf STATUS,C
              rlf BASS,w
              addlw BASS_TABLE-BASE1
              call MORO2

              movwf BUFFERIIC+4

              bcf STATUS,C
              rlf BASS,w
              addlw .1
              addlw BASS_TABLE-BASE1
              call MORO2
               
              movwf BUFFERIIC+5
              
              call SEND_BUFFER              

              return
;**********************************************************
;**********************************************************               
TREBLE_MAS
              movf TREBLE,w
              movwf PRE
              call PRE_MAS

              movlw .6
              movwf N_BYTEIIC
              movlw 0x3A
              movwf BUFFERIIC
              movlw 0x68
              movwf BUFFERIIC+1
              movlw 0x96
              movwf BUFFERIIC+2
              movlw 0xF0
              movwf BUFFERIIC+3
             
              bcf STATUS,C
              rlf TREBLE,w
              addlw TREBLE_TABLE-BASE1
              call MORO2

              movwf BUFFERIIC+4

              bcf STATUS,C
              rlf TREBLE,w
              addlw .1
              addlw TREBLE_TABLE-BASE1
              call MORO2
               
              movwf BUFFERIIC+5
              
              call SEND_BUFFER

              return
;**********************************************************
;**********************************************************               
PRE_MAS
              movlw .6
              movwf N_BYTEIIC
              movlw 0x3A
              movwf BUFFERIIC
              movlw 0x68
              movwf BUFFERIIC+1
              movlw 0x9E
              movwf BUFFERIIC+2
              movlw 0x70
              movwf BUFFERIIC+3
             
              bcf STATUS,C
              rlf PRE,w
              addlw PRE_TABLE-BASE1
              call MORO2

              movwf BUFFERIIC+4

              bcf STATUS,C
              rlf PRE,w
              addlw .1
              addlw PRE_TABLE-BASE1
              call MORO2
               
              movwf BUFFERIIC+5
              
              call SEND_BUFFER

              return
;**********************************************************
;**********************************************************
MORO2   
              movwf TEMPW
              movlw 0x0E
              movwf PCLATH
              movf TEMPW,w
              movwf PCL                                         
;**********************************************************
;**********************************************************
SEND_BUFFER
	      clrf I2C_FAIL
              call INI_IIC 

              call START_IIC
               
              movlw BUFFERIIC
              movwf FSR
OTROFUERA
              movf INDF,w
              movwf BYTEIIC
 
              call OUT_IIC

              call GETACK_IIC
	      btfsc I2C_FAIL,0
	      goto FAILED_SO_ABORT

              incf FSR
              decfsz N_BYTEIIC               
              goto OTROFUERA
FAILED_SO_ABORT
              call STOP_IIC

              return
;*********************************
;*********************************
INI_IIC 
              bsf STATUS,RP0
              bcf TRISA,SCL
              bcf TRISA,SDA
              bcf STATUS,RP0
            
              clrf PORTA
              return 
;*********************************
;*********************************
START_IIC
              sdaup               
              nop
              nop
              nop
              sclup
              nop
              nop
              nop
              sdadown
              nop
              nop
              nop
              scldown
              nop
              nop
              nop
              return
;*********************************
;*********************************
STOP_IIC
              sdadown
              nop
              nop
              nop
              sclup
              nop
              nop
              nop
              sdaup
              nop
              nop
              nop
              return
;*********************************
;*********************************
GETACK_IIC
              sdaup
              nop
              nop
              nop
HMMS
              sclup
              btfss PORTA,SCL
              goto HMMS

              bcf FLAG,ACK
              btfsc PORTA,SDA
              incf I2C_FAIL
              nop
              nop
              nop
              scldown
              return
;*********************************
;*********************************
OUT_IIC
              movlw .8
              movwf COUNTER
LOOP11
              nop
              nop
              nop
              btfsc BYTEIIC,7
              goto UPP
              sdadown
              goto HMMS1
UPP
              sdaup 
HMMS1
              nop
              nop
              nop
              sclup
              btfss PORTA,SCL
              goto HMMS1
              rlf BYTEIIC
              nop
              nop
              nop
              scldown
              decfsz COUNTER
              goto LOOP11
              return

;**********************************************************
;**********************************************************
          ; Tablas 1 (Volumen, Bass, Treble, Prefactor tables)
BASE1         equ 0xE00
               
              org 0xE00
VOLUMEN_TABLE
               ; 0  1   2   3   4   5   6   7   8   9   10  11  12  13  14  15
              DT 0,.12,.20,.28,.32,.36,.38,.40,.42,.44,.46,.48,.50,.52,.54,.56 
BASS_TABLE
               ;     0         1         2         3         4         5
              DT 0x9e,0x40,0xa7,0x40,0xb1,0x80,0xbd,0x40,0xca,0x40,0xd8,0xc0
               ;     6         7      8      9        10        11
              DT 0xe8,0x00,0xf7,0xc0,0,0,0x00,0x80,0x17,0xc0,0x27,0x00
               ;    12        13        14        15        16
	      DT 0x35,0x80,0x42,0x80,0x4e,0x40,0x58,0x80,0x61,0x80                                     

TREBLE_TABLE
               ;     0         1         2         3         4         5
              DT 0xb2,0xc0,0xc1,0x80,0xcb,0xc0,0xd5,0x00,0xde,0x00,0xe7,0xe0             
               ;     6         7      8      9        10        11
              DT 0xf0,0xc0,0xfa,0xc0,0,0,0x05,0x40,0x10,0x40,0x1c,0x00 
               ;    12        13        14        15        16
              DT 0x28,0x40,0x35,0x40,0x42,0xc0,0x51,0x80,0x5f,0x80

PRE_TABLE
               ;     0         1         2         3         4         5
              DT 0x80,0x00,0x80,0x00,0x80,0x00,0x80,0x00,0x80,0x00,0x80,0x00
               ;     6         7         8         9        10        11
              DT 0x80,0x00,0x80,0x00,0x80,0x00,0x8e,0x00,0xa5,0x80,0xb8,0x00
               ;    12        13        14        15        16
              DT 0xc6,0xc0,0xd2,0x50,0xdc,0x00,0xe3,0x40,0xe9,0x40
;**********************************************************
;**********************************************************
          ; Tablas 2 (texts and Iso9660 mark)

BASE2         equ 0xF00
               
              org 0xF00
M_ID
              retlw 0x02
              DT "CD001"
M_TRACK
              DT "Track"       
M_NUM
              DT "0123456789"
M_0
              DT " MP3 Public CD  " ;1  
M_1
              DT "  No CD Found   " ;
M_2
              DT "     CD-DA      " ;2
M_3
              DT "     CD-MP3     " ;3
M_4
              DT "   CD-Hybrid    " ;4
M_5
              DT "Tracks Found:   " ;5
M_6
              DT "    Volume      " ;6
M_7
              DT "     Bass       " ;7  
M_8     
              DT "    Treble      " ;0 
M_9
              DT "                " ;1
M_10     
              DT "ATAPI Not Found " ;
M_11
              DT "   Door Open    " ;2

PLL_TABLE				; These values are my calculations and could be wrong?
		addwf		PCL,1	;  Values must be written backwards
#ifdef XTAL_14.31818Hz
		dt	0x0C,0x00,0xEB,0xEC,0x05,0x00,0xD1,0xD9,0x2D,0x03,0x02,0x00,0x00,0xA0,0x68,0x3A
#endif
#ifdef XTAL_14.745MHz
		dt	0x0A,0x00,0x05,0x01,0x02,0x00,0xC8,0x2A,0x2D,0x03,0x02,0x00,0x00,0xA0,0x68,0x3A
#endif